```@meta
EditURL = "https://github.com/maartenvd/MPSKit.jl/examples/quantum1d/1.ising-cft/main.jl"
```

[![](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/maartenvd/MPSKit.jl/gh-pages?filepath=dev/examples/quantum1d/1.ising-cft/main.ipynb)
[![](https://img.shields.io/badge/show-nbviewer-579ACA.svg)](https://nbviewer.jupyter.org/github/maartenvd/MPSKit.jl/blob/gh-pages/dev/examples/quantum1d/1.ising-cft/main.ipynb)
[![](https://img.shields.io/badge/download-project-orange)](https://minhaskamal.github.io/DownGit/#/home?url=https://github.com/maartenvd/MPSKit.jl/examples/tree/gh-pages/dev/examples/quantum1d/1.ising-cft)

# The Ising CFT spectrum

This tutorial is meant to show the finite size CFT spectrum for the quantum Ising model. We
do this by first employing an exact diagonalization technique, and then extending the
analysis to larger system sizes through the use of MPS techniques.

````julia
using MPSKit, MPSKitModels, TensorKit, Plots, KrylovKit
using LinearAlgebra: eigen, diagm, Hermitian
import TensorOperations; TensorOperations.disable_cache(); # hide
````

The hamiltonian is defined on a finite lattice with periodic boundary conditions,
which can be implemented as follows:

````julia
L = 12
H = periodic_boundary_conditions(transverse_field_ising(), L);
````

````
┌ Warning: failed to deduce all domspaces
└ @ MPSKit ~/projects/MPSKit.jl/src/operators/sparsempo/sparsempo.jl:144

````

## Exact diagonalisation

In MPSKit, there is support for exact diagonalisation by leveraging the fact that applying
the hamiltonian to an untruncated MPS will result in an effective hamiltonian on the center
site which implements the action of the entire hamiltonian. Thus, optimizing the middle
tensor is equivalent to optimixing a state in the entire Hilbert space, as all other tensors
are just unitary matrices that mix the basis.

````julia
energies, states = exact_diagonalization(H, num=18, alg=Lanczos(krylovdim=200));
plot(real.(energies), seriestype=:scatter, legend=false, ylabel="energy", xlabel="#eigenvalue")
````

```@raw html
<?xml version="1.0" encoding="utf-8"?>
<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="600" height="400" viewBox="0 0 2400 1600">
<defs>
  <clipPath id="clip250">
    <rect x="0" y="0" width="2400" height="1600"/>
  </clipPath>
</defs>
<path clip-path="url(#clip250)" d="M0 1600 L2400 1600 L2400 0 L0 0  Z" fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip251">
    <rect x="480" y="0" width="1681" height="1600"/>
  </clipPath>
</defs>
<path clip-path="url(#clip250)" d="M242.621 1423.18 L2352.76 1423.18 L2352.76 47.2441 L242.621 47.2441  Z" fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip252">
    <rect x="242" y="47" width="2111" height="1377"/>
  </clipPath>
</defs>
<polyline clip-path="url(#clip252)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="620.853,1423.18 620.853,47.2441 "/>
<polyline clip-path="url(#clip252)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1018.99,1423.18 1018.99,47.2441 "/>
<polyline clip-path="url(#clip252)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1417.13,1423.18 1417.13,47.2441 "/>
<polyline clip-path="url(#clip252)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1815.27,1423.18 1815.27,47.2441 "/>
<polyline clip-path="url(#clip252)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="2213.41,1423.18 2213.41,47.2441 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="242.621,1423.18 2352.76,1423.18 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="620.853,1423.18 620.853,1404.28 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1018.99,1423.18 1018.99,1404.28 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1417.13,1423.18 1417.13,1404.28 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1815.27,1423.18 1815.27,1404.28 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="2213.41,1423.18 2213.41,1404.28 "/>
<path clip-path="url(#clip250)" d="M611.13 1451.02 L629.487 1451.02 L629.487 1454.96 L615.413 1454.96 L615.413 1463.43 Q616.431 1463.08 617.45 1462.92 Q618.468 1462.73 619.487 1462.73 Q625.274 1462.73 628.653 1465.9 Q632.033 1469.08 632.033 1474.49 Q632.033 1480.07 628.561 1483.17 Q625.089 1486.25 618.769 1486.25 Q616.593 1486.25 614.325 1485.88 Q612.079 1485.51 609.672 1484.77 L609.672 1480.07 Q611.755 1481.2 613.978 1481.76 Q616.2 1482.32 618.677 1482.32 Q622.681 1482.32 625.019 1480.21 Q627.357 1478.1 627.357 1474.49 Q627.357 1470.88 625.019 1468.77 Q622.681 1466.67 618.677 1466.67 Q616.802 1466.67 614.927 1467.08 Q613.075 1467.5 611.13 1468.38 L611.13 1451.02 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M993.679 1481.64 L1001.32 1481.64 L1001.32 1455.28 L993.008 1456.95 L993.008 1452.69 L1001.27 1451.02 L1005.95 1451.02 L1005.95 1481.64 L1013.59 1481.64 L1013.59 1485.58 L993.679 1485.58 L993.679 1481.64 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1033.03 1454.1 Q1029.42 1454.1 1027.59 1457.66 Q1025.79 1461.2 1025.79 1468.33 Q1025.79 1475.44 1027.59 1479.01 Q1029.42 1482.55 1033.03 1482.55 Q1036.66 1482.55 1038.47 1479.01 Q1040.3 1475.44 1040.3 1468.33 Q1040.3 1461.2 1038.47 1457.66 Q1036.66 1454.1 1033.03 1454.1 M1033.03 1450.39 Q1038.84 1450.39 1041.9 1455 Q1044.97 1459.58 1044.97 1468.33 Q1044.97 1477.06 1041.9 1481.67 Q1038.84 1486.25 1033.03 1486.25 Q1027.22 1486.25 1024.14 1481.67 Q1021.09 1477.06 1021.09 1468.33 Q1021.09 1459.58 1024.14 1455 Q1027.22 1450.39 1033.03 1450.39 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1392.32 1481.64 L1399.95 1481.64 L1399.95 1455.28 L1391.64 1456.95 L1391.64 1452.69 L1399.91 1451.02 L1404.58 1451.02 L1404.58 1481.64 L1412.22 1481.64 L1412.22 1485.58 L1392.32 1485.58 L1392.32 1481.64 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1421.71 1451.02 L1440.07 1451.02 L1440.07 1454.96 L1426 1454.96 L1426 1463.43 Q1427.01 1463.08 1428.03 1462.92 Q1429.05 1462.73 1430.07 1462.73 Q1435.86 1462.73 1439.24 1465.9 Q1442.62 1469.08 1442.62 1474.49 Q1442.62 1480.07 1439.14 1483.17 Q1435.67 1486.25 1429.35 1486.25 Q1427.18 1486.25 1424.91 1485.88 Q1422.66 1485.51 1420.25 1484.77 L1420.25 1480.07 Q1422.34 1481.2 1424.56 1481.76 Q1426.78 1482.32 1429.26 1482.32 Q1433.26 1482.32 1435.6 1480.21 Q1437.94 1478.1 1437.94 1474.49 Q1437.94 1470.88 1435.6 1468.77 Q1433.26 1466.67 1429.26 1466.67 Q1427.38 1466.67 1425.51 1467.08 Q1423.66 1467.5 1421.71 1468.38 L1421.71 1451.02 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1794.04 1481.64 L1810.36 1481.64 L1810.36 1485.58 L1788.42 1485.58 L1788.42 1481.64 Q1791.08 1478.89 1795.66 1474.26 Q1800.27 1469.61 1801.45 1468.27 Q1803.69 1465.74 1804.57 1464.01 Q1805.48 1462.25 1805.48 1460.56 Q1805.48 1457.8 1803.53 1456.07 Q1801.61 1454.33 1798.51 1454.33 Q1796.31 1454.33 1793.86 1455.09 Q1791.43 1455.86 1788.65 1457.41 L1788.65 1452.69 Q1791.47 1451.55 1793.93 1450.97 Q1796.38 1450.39 1798.42 1450.39 Q1803.79 1450.39 1806.98 1453.08 Q1810.18 1455.77 1810.18 1460.26 Q1810.18 1462.39 1809.37 1464.31 Q1808.58 1466.2 1806.47 1468.8 Q1805.89 1469.47 1802.79 1472.69 Q1799.69 1475.88 1794.04 1481.64 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1830.18 1454.1 Q1826.56 1454.1 1824.74 1457.66 Q1822.93 1461.2 1822.93 1468.33 Q1822.93 1475.44 1824.74 1479.01 Q1826.56 1482.55 1830.18 1482.55 Q1833.81 1482.55 1835.62 1479.01 Q1837.44 1475.44 1837.44 1468.33 Q1837.44 1461.2 1835.62 1457.66 Q1833.81 1454.1 1830.18 1454.1 M1830.18 1450.39 Q1835.99 1450.39 1839.04 1455 Q1842.12 1459.58 1842.12 1468.33 Q1842.12 1477.06 1839.04 1481.67 Q1835.99 1486.25 1830.18 1486.25 Q1824.37 1486.25 1821.29 1481.67 Q1818.23 1477.06 1818.23 1468.33 Q1818.23 1459.58 1821.29 1455 Q1824.37 1450.39 1830.18 1450.39 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M2192.68 1481.64 L2209 1481.64 L2209 1485.58 L2187.05 1485.58 L2187.05 1481.64 Q2189.72 1478.89 2194.3 1474.26 Q2198.91 1469.61 2200.09 1468.27 Q2202.33 1465.74 2203.21 1464.01 Q2204.11 1462.25 2204.11 1460.56 Q2204.11 1457.8 2202.17 1456.07 Q2200.25 1454.33 2197.15 1454.33 Q2194.95 1454.33 2192.49 1455.09 Q2190.06 1455.86 2187.28 1457.41 L2187.28 1452.69 Q2190.11 1451.55 2192.56 1450.97 Q2195.02 1450.39 2197.05 1450.39 Q2202.42 1450.39 2205.62 1453.08 Q2208.81 1455.77 2208.81 1460.26 Q2208.81 1462.39 2208 1464.31 Q2207.22 1466.2 2205.11 1468.8 Q2204.53 1469.47 2201.43 1472.69 Q2198.33 1475.88 2192.68 1481.64 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M2218.86 1451.02 L2237.22 1451.02 L2237.22 1454.96 L2223.14 1454.96 L2223.14 1463.43 Q2224.16 1463.08 2225.18 1462.92 Q2226.2 1462.73 2227.22 1462.73 Q2233 1462.73 2236.38 1465.9 Q2239.76 1469.08 2239.76 1474.49 Q2239.76 1480.07 2236.29 1483.17 Q2232.82 1486.25 2226.5 1486.25 Q2224.32 1486.25 2222.05 1485.88 Q2219.81 1485.51 2217.4 1484.77 L2217.4 1480.07 Q2219.48 1481.2 2221.71 1481.76 Q2223.93 1482.32 2226.4 1482.32 Q2230.41 1482.32 2232.75 1480.21 Q2235.09 1478.1 2235.09 1474.49 Q2235.09 1470.88 2232.75 1468.77 Q2230.41 1466.67 2226.4 1466.67 Q2224.53 1466.67 2222.65 1467.08 Q2220.8 1467.5 2218.86 1468.38 L2218.86 1451.02 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1123.4 1539.37 L1114.13 1539.37 L1111.46 1550 L1120.79 1550 L1123.4 1539.37 M1118.62 1521.26 L1115.31 1534.46 L1124.6 1534.46 L1127.95 1521.26 L1133.04 1521.26 L1129.76 1534.46 L1139.69 1534.46 L1139.69 1539.37 L1128.52 1539.37 L1125.91 1550 L1136.03 1550 L1136.03 1554.87 L1124.67 1554.87 L1121.36 1568.04 L1116.27 1568.04 L1119.54 1554.87 L1110.22 1554.87 L1106.94 1568.04 L1101.82 1568.04 L1105.13 1554.87 L1095.1 1554.87 L1095.1 1550 L1106.3 1550 L1108.98 1539.37 L1098.73 1539.37 L1098.73 1534.46 L1110.22 1534.46 L1113.46 1521.26 L1118.62 1521.26 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1181.32 1548.76 L1181.32 1551.62 L1154.4 1551.62 Q1154.78 1557.67 1158.02 1560.85 Q1161.3 1564 1167.13 1564 Q1170.5 1564 1173.65 1563.17 Q1176.84 1562.35 1179.95 1560.69 L1179.95 1566.23 Q1176.8 1567.57 1173.49 1568.27 Q1170.18 1568.97 1166.78 1568.97 Q1158.25 1568.97 1153.25 1564 Q1148.29 1559.04 1148.29 1550.57 Q1148.29 1541.82 1153 1536.69 Q1157.74 1531.54 1165.76 1531.54 Q1172.95 1531.54 1177.12 1536.18 Q1181.32 1540.8 1181.32 1548.76 M1175.47 1547.04 Q1175.4 1542.23 1172.76 1539.37 Q1170.15 1536.5 1165.82 1536.5 Q1160.92 1536.5 1157.96 1539.27 Q1155.03 1542.04 1154.59 1547.07 L1175.47 1547.04 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1190.94 1532.4 L1196.79 1532.4 L1196.79 1568.04 L1190.94 1568.04 L1190.94 1532.4 M1190.94 1518.52 L1196.79 1518.52 L1196.79 1525.93 L1190.94 1525.93 L1190.94 1518.52 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1232.5 1549.81 Q1232.5 1543.44 1229.86 1539.94 Q1227.25 1536.44 1222.51 1536.44 Q1217.8 1536.44 1215.16 1539.94 Q1212.55 1543.44 1212.55 1549.81 Q1212.55 1556.14 1215.16 1559.64 Q1217.8 1563.14 1222.51 1563.14 Q1227.25 1563.14 1229.86 1559.64 Q1232.5 1556.14 1232.5 1549.81 M1238.36 1563.62 Q1238.36 1572.72 1234.32 1577.15 Q1230.28 1581.6 1221.94 1581.6 Q1218.85 1581.6 1216.11 1581.13 Q1213.37 1580.68 1210.8 1579.72 L1210.8 1574.03 Q1213.37 1575.43 1215.89 1576.1 Q1218.4 1576.76 1221.01 1576.76 Q1226.77 1576.76 1229.64 1573.74 Q1232.5 1570.75 1232.5 1564.67 L1232.5 1561.77 Q1230.69 1564.92 1227.86 1566.48 Q1225.02 1568.04 1221.08 1568.04 Q1214.52 1568.04 1210.51 1563.05 Q1206.5 1558.05 1206.5 1549.81 Q1206.5 1541.53 1210.51 1536.53 Q1214.52 1531.54 1221.08 1531.54 Q1225.02 1531.54 1227.86 1533.1 Q1230.69 1534.66 1232.5 1537.81 L1232.5 1532.4 L1238.36 1532.4 L1238.36 1563.62 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1280.91 1548.76 L1280.91 1551.62 L1253.99 1551.62 Q1254.37 1557.67 1257.62 1560.85 Q1260.89 1564 1266.72 1564 Q1270.09 1564 1273.24 1563.17 Q1276.43 1562.35 1279.55 1560.69 L1279.55 1566.23 Q1276.4 1567.57 1273.08 1568.27 Q1269.77 1568.97 1266.37 1568.97 Q1257.84 1568.97 1252.84 1564 Q1247.88 1559.04 1247.88 1550.57 Q1247.88 1541.82 1252.59 1536.69 Q1257.33 1531.54 1265.35 1531.54 Q1272.54 1531.54 1276.71 1536.18 Q1280.91 1540.8 1280.91 1548.76 M1275.06 1547.04 Q1274.99 1542.23 1272.35 1539.37 Q1269.74 1536.5 1265.41 1536.5 Q1260.51 1536.5 1257.55 1539.27 Q1254.62 1542.04 1254.18 1547.07 L1275.06 1547.04 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1320.16 1546.53 L1320.16 1568.04 L1314.3 1568.04 L1314.3 1546.72 Q1314.3 1541.66 1312.33 1539.14 Q1310.36 1536.63 1306.41 1536.63 Q1301.67 1536.63 1298.93 1539.65 Q1296.19 1542.68 1296.19 1547.9 L1296.19 1568.04 L1290.3 1568.04 L1290.3 1532.4 L1296.19 1532.4 L1296.19 1537.93 Q1298.29 1534.72 1301.13 1533.13 Q1303.99 1531.54 1307.71 1531.54 Q1313.86 1531.54 1317.01 1535.36 Q1320.16 1539.14 1320.16 1546.53 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1327.64 1532.4 L1333.85 1532.4 L1344.99 1562.31 L1356.13 1532.4 L1362.33 1532.4 L1348.96 1568.04 L1341.01 1568.04 L1327.64 1532.4 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1386.62 1550.12 Q1379.52 1550.12 1376.78 1551.75 Q1374.04 1553.37 1374.04 1557.29 Q1374.04 1560.4 1376.08 1562.25 Q1378.15 1564.07 1381.68 1564.07 Q1386.55 1564.07 1389.48 1560.63 Q1392.44 1557.16 1392.44 1551.43 L1392.44 1550.12 L1386.62 1550.12 M1398.3 1547.71 L1398.3 1568.04 L1392.44 1568.04 L1392.44 1562.63 Q1390.44 1565.88 1387.44 1567.44 Q1384.45 1568.97 1380.12 1568.97 Q1374.65 1568.97 1371.4 1565.91 Q1368.19 1562.82 1368.19 1557.67 Q1368.19 1551.65 1372.2 1548.6 Q1376.24 1545.54 1384.23 1545.54 L1392.44 1545.54 L1392.44 1544.97 Q1392.44 1540.93 1389.77 1538.73 Q1387.13 1536.5 1382.32 1536.5 Q1379.26 1536.5 1376.37 1537.23 Q1373.47 1537.97 1370.8 1539.43 L1370.8 1534.02 Q1374.01 1532.78 1377.04 1532.17 Q1380.06 1531.54 1382.93 1531.54 Q1390.66 1531.54 1394.48 1535.55 Q1398.3 1539.56 1398.3 1547.71 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1410.36 1518.52 L1416.22 1518.52 L1416.22 1568.04 L1410.36 1568.04 L1410.36 1518.52 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1427.87 1553.98 L1427.87 1532.4 L1433.72 1532.4 L1433.72 1553.75 Q1433.72 1558.81 1435.7 1561.36 Q1437.67 1563.87 1441.62 1563.87 Q1446.36 1563.87 1449.1 1560.85 Q1451.87 1557.83 1451.87 1552.61 L1451.87 1532.4 L1457.72 1532.4 L1457.72 1568.04 L1451.87 1568.04 L1451.87 1562.57 Q1449.73 1565.82 1446.9 1567.41 Q1444.1 1568.97 1440.38 1568.97 Q1434.23 1568.97 1431.05 1565.15 Q1427.87 1561.33 1427.87 1553.98 M1442.6 1531.54 L1442.6 1531.54 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M1500.28 1548.76 L1500.28 1551.62 L1473.35 1551.62 Q1473.73 1557.67 1476.98 1560.85 Q1480.26 1564 1486.08 1564 Q1489.46 1564 1492.61 1563.17 Q1495.79 1562.35 1498.91 1560.69 L1498.91 1566.23 Q1495.76 1567.57 1492.45 1568.27 Q1489.14 1568.97 1485.73 1568.97 Q1477.2 1568.97 1472.2 1564 Q1467.24 1559.04 1467.24 1550.57 Q1467.24 1541.82 1471.95 1536.69 Q1476.69 1531.54 1484.71 1531.54 Q1491.91 1531.54 1496.08 1536.18 Q1500.28 1540.8 1500.28 1548.76 M1494.42 1547.04 Q1494.36 1542.23 1491.72 1539.37 Q1489.11 1536.5 1484.78 1536.5 Q1479.87 1536.5 1476.91 1539.27 Q1473.99 1542.04 1473.54 1547.07 L1494.42 1547.04 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><polyline clip-path="url(#clip252)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="242.621,1270.93 2352.76,1270.93 "/>
<polyline clip-path="url(#clip252)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="242.621,919.676 2352.76,919.676 "/>
<polyline clip-path="url(#clip252)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="242.621,568.426 2352.76,568.426 "/>
<polyline clip-path="url(#clip252)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="242.621,217.175 2352.76,217.175 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="242.621,1423.18 242.621,47.2441 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="242.621,1270.93 261.518,1270.93 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="242.621,919.676 261.518,919.676 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="242.621,568.426 261.518,568.426 "/>
<polyline clip-path="url(#clip250)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="242.621,217.175 261.518,217.175 "/>
<path clip-path="url(#clip250)" d="M115.742 1271.38 L145.417 1271.38 L145.417 1275.31 L115.742 1275.31 L115.742 1271.38 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M156.32 1284.27 L163.959 1284.27 L163.959 1257.91 L155.649 1259.57 L155.649 1255.31 L163.913 1253.65 L168.589 1253.65 L168.589 1284.27 L176.227 1284.27 L176.227 1288.21 L156.32 1288.21 L156.32 1284.27 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M185.718 1253.65 L204.074 1253.65 L204.074 1257.58 L190 1257.58 L190 1266.05 Q191.019 1265.71 192.038 1265.54 Q193.056 1265.36 194.075 1265.36 Q199.862 1265.36 203.241 1268.53 Q206.621 1271.7 206.621 1277.12 Q206.621 1282.7 203.149 1285.8 Q199.676 1288.88 193.357 1288.88 Q191.181 1288.88 188.913 1288.51 Q186.667 1288.14 184.26 1287.4 L184.26 1282.7 Q186.343 1283.83 188.565 1284.39 Q190.788 1284.94 193.264 1284.94 Q197.269 1284.94 199.607 1282.84 Q201.945 1280.73 201.945 1277.12 Q201.945 1273.51 199.607 1271.4 Q197.269 1269.29 193.264 1269.29 Q191.389 1269.29 189.514 1269.71 Q187.663 1270.13 185.718 1271.01 L185.718 1253.65 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M114.26 920.127 L143.936 920.127 L143.936 924.063 L114.26 924.063 L114.26 920.127 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M154.839 933.021 L162.477 933.021 L162.477 906.655 L154.167 908.322 L154.167 904.063 L162.431 902.396 L167.107 902.396 L167.107 933.021 L174.746 933.021 L174.746 936.956 L154.839 936.956 L154.839 933.021 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M197.037 906.47 L185.232 924.919 L197.037 924.919 L197.037 906.47 M195.811 902.396 L201.69 902.396 L201.69 924.919 L206.621 924.919 L206.621 928.808 L201.69 928.808 L201.69 936.956 L197.037 936.956 L197.037 928.808 L181.436 928.808 L181.436 924.294 L195.811 902.396 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M115.394 568.877 L145.07 568.877 L145.07 572.812 L115.394 572.812 L115.394 568.877 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M155.973 581.77 L163.612 581.77 L163.612 555.405 L155.302 557.071 L155.302 552.812 L163.565 551.146 L168.241 551.146 L168.241 581.77 L175.88 581.77 L175.88 585.706 L155.973 585.706 L155.973 581.77 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M199.491 567.071 Q202.848 567.789 204.723 570.057 Q206.621 572.326 206.621 575.659 Q206.621 580.775 203.102 583.576 Q199.584 586.377 193.102 586.377 Q190.926 586.377 188.612 585.937 Q186.32 585.52 183.866 584.664 L183.866 580.15 Q185.811 581.284 188.126 581.863 Q190.44 582.442 192.963 582.442 Q197.362 582.442 199.653 580.706 Q201.968 578.969 201.968 575.659 Q201.968 572.604 199.815 570.891 Q197.686 569.155 193.866 569.155 L189.838 569.155 L189.838 565.312 L194.051 565.312 Q197.5 565.312 199.329 563.946 Q201.158 562.558 201.158 559.965 Q201.158 557.303 199.26 555.891 Q197.385 554.456 193.866 554.456 Q191.945 554.456 189.746 554.872 Q187.547 555.289 184.908 556.169 L184.908 552.002 Q187.57 551.261 189.885 550.891 Q192.223 550.521 194.283 550.521 Q199.607 550.521 202.709 552.951 Q205.811 555.358 205.811 559.479 Q205.811 562.349 204.167 564.34 Q202.524 566.308 199.491 567.071 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M116.343 217.626 L146.019 217.626 L146.019 221.562 L116.343 221.562 L116.343 217.626 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M156.922 230.52 L164.561 230.52 L164.561 204.154 L156.251 205.821 L156.251 201.562 L164.515 199.895 L169.19 199.895 L169.19 230.52 L176.829 230.52 L176.829 234.455 L156.922 234.455 L156.922 230.52 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M190.301 230.52 L206.621 230.52 L206.621 234.455 L184.676 234.455 L184.676 230.52 Q187.338 227.765 191.922 223.136 Q196.528 218.483 197.709 217.14 Q199.954 214.617 200.834 212.881 Q201.737 211.122 201.737 209.432 Q201.737 206.677 199.792 204.941 Q197.871 203.205 194.769 203.205 Q192.57 203.205 190.116 203.969 Q187.686 204.733 184.908 206.284 L184.908 201.562 Q187.732 200.427 190.186 199.849 Q192.639 199.27 194.676 199.27 Q200.047 199.27 203.241 201.955 Q206.436 204.64 206.436 209.131 Q206.436 211.261 205.625 213.182 Q204.838 215.08 202.732 217.673 Q202.153 218.344 199.051 221.562 Q195.95 224.756 190.301 230.52 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M44.7161 812.969 L47.5806 812.969 L47.5806 839.896 Q53.6281 839.514 56.8109 836.268 Q59.9619 832.989 59.9619 827.165 Q59.9619 823.791 59.1344 820.64 Q58.3069 817.457 56.6518 814.338 L62.1899 814.338 Q63.5267 817.489 64.227 820.799 Q64.9272 824.109 64.9272 827.515 Q64.9272 836.045 59.9619 841.042 Q54.9967 846.007 46.5303 846.007 Q37.7774 846.007 32.6531 841.296 Q27.4968 836.554 27.4968 828.533 Q27.4968 821.34 32.1438 817.17 Q36.7589 812.969 44.7161 812.969 M42.9973 818.826 Q38.1912 818.889 35.3266 821.531 Q32.4621 824.141 32.4621 828.47 Q32.4621 833.371 35.2312 836.331 Q38.0002 839.259 43.0292 839.705 L42.9973 818.826 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M42.4881 773.724 L64.0042 773.724 L64.0042 779.581 L42.679 779.581 Q37.6183 779.581 35.1038 781.554 Q32.5894 783.528 32.5894 787.474 Q32.5894 792.217 35.6131 794.954 Q38.6368 797.691 43.8567 797.691 L64.0042 797.691 L64.0042 803.58 L28.3562 803.58 L28.3562 797.691 L33.8944 797.691 Q30.6797 795.591 29.0883 792.758 Q27.4968 789.893 27.4968 786.169 Q27.4968 780.027 31.3163 776.876 Q35.1038 773.724 42.4881 773.724 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M44.7161 731.552 L47.5806 731.552 L47.5806 758.479 Q53.6281 758.097 56.8109 754.85 Q59.9619 751.572 59.9619 745.747 Q59.9619 742.373 59.1344 739.222 Q58.3069 736.04 56.6518 732.92 L62.1899 732.92 Q63.5267 736.071 64.227 739.382 Q64.9272 742.692 64.9272 746.097 Q64.9272 754.627 59.9619 759.624 Q54.9967 764.59 46.5303 764.59 Q37.7774 764.59 32.6531 759.879 Q27.4968 755.137 27.4968 747.116 Q27.4968 739.923 32.1438 735.753 Q36.7589 731.552 44.7161 731.552 M42.9973 737.408 Q38.1912 737.472 35.3266 740.114 Q32.4621 742.724 32.4621 747.052 Q32.4621 751.954 35.2312 754.914 Q38.0002 757.842 43.0292 758.288 L42.9973 737.408 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M33.8307 701.283 Q33.2578 702.269 33.0032 703.447 Q32.7167 704.593 32.7167 705.993 Q32.7167 710.959 35.9632 713.632 Q39.1779 716.274 45.2253 716.274 L64.0042 716.274 L64.0042 722.162 L28.3562 722.162 L28.3562 716.274 L33.8944 716.274 Q30.6479 714.428 29.0883 711.468 Q27.4968 708.508 27.4968 704.275 Q27.4968 703.67 27.5923 702.938 Q27.656 702.206 27.8151 701.315 L33.8307 701.283 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M45.7664 672.828 Q39.4007 672.828 35.8996 675.47 Q32.3984 678.08 32.3984 682.822 Q32.3984 687.533 35.8996 690.175 Q39.4007 692.785 45.7664 692.785 Q52.1003 692.785 55.6014 690.175 Q59.1026 687.533 59.1026 682.822 Q59.1026 678.08 55.6014 675.47 Q52.1003 672.828 45.7664 672.828 M59.58 666.972 Q68.683 666.972 73.1071 671.014 Q77.5631 675.056 77.5631 683.395 Q77.5631 686.482 77.0857 689.22 Q76.6401 691.957 75.6852 694.535 L69.9879 694.535 Q71.3884 691.957 72.0568 689.443 Q72.7252 686.928 72.7252 684.318 Q72.7252 678.557 69.7015 675.693 Q66.7096 672.828 60.6303 672.828 L57.7339 672.828 Q60.885 674.642 62.4446 677.475 Q64.0042 680.308 64.0042 684.254 Q64.0042 690.811 59.0071 694.822 Q54.01 698.832 45.7664 698.832 Q37.491 698.832 32.4939 694.822 Q27.4968 690.811 27.4968 684.254 Q27.4968 680.308 29.0564 677.475 Q30.616 674.642 33.7671 672.828 L28.3562 672.828 L28.3562 666.972 L59.58 666.972 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip250)" d="M67.3143 640.076 Q73.68 642.559 75.6216 644.914 Q77.5631 647.27 77.5631 651.216 L77.5631 655.895 L72.6615 655.895 L72.6615 652.458 Q72.6615 650.039 71.5157 648.702 Q70.3699 647.365 66.1048 645.742 L63.4312 644.692 L28.3562 659.11 L28.3562 652.903 L56.238 641.763 L28.3562 630.623 L28.3562 624.417 L67.3143 640.076 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><circle clip-path="url(#clip252)" cx="302.342" cy="1384.24" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="381.969" cy="1338.19" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="461.597" cy="1017.46" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="541.225" cy="974.553" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="620.853" cy="974.553" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="700.48" cy="663.178" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="780.108" cy="663.178" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="859.736" cy="663.178" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="939.364" cy="663.178" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1018.99" cy="635.693" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1098.62" cy="635.693" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1178.25" cy="610.911" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1257.87" cy="345.538" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1337.5" cy="345.538" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1417.13" cy="345.538" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1496.76" cy="345.538" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1576.39" cy="344.707" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1656.01" cy="344.707" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1735.64" cy="308.896" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1815.27" cy="272.052" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1894.9" cy="272.052" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="1974.52" cy="272.052" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="2054.15" cy="272.052" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="2133.78" cy="121.426" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="2213.41" cy="121.426" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip252)" cx="2293.04" cy="86.1857" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
</svg>

```

!!! note "Krylov dimension"
    Note that we have specified a large Krylov dimension as degenerate eigenvalues are
    notoriously difficult for iterative methods.

## Extracting momentum

Given a state, it is possible to assign a momentum label
through the use of the translation operator. This operator can be defined in MPO language
either diagramatically as

![translation operator MPO](translation_mpo.png)

or in the code as:

````julia
id = complex(isomorphism(ℂ^2, ℂ^2))
@tensor O[-1 -2; -3 -4] := id[-1, -3] * id[-2, -4]
T = periodic_boundary_conditions(DenseMPO(O), L);
````

We can then calculate the momentum of the groundstate as the expectation value of this
operator. However, there is a subtlety because of the degeneracies in the energy
eigenvalues. The eigensolver will find an orthonormal basis within each energy subspace, but
this basis is not necessarily a basis of eigenstates of the translation operator. In order
to fix this, we diagonalize the translation operator within each energy subspace.

````julia
momentum(ψᵢ, ψⱼ=ψᵢ) = angle(dot(ψᵢ, T * ψⱼ))

function fix_degeneracies(basis)
    N = zeros(ComplexF64, length(basis), length(basis))
    M = zeros(ComplexF64, length(basis), length(basis))
    for i in eachindex(basis), j in eachindex(basis)
        N[i, j] = dot(basis[i], basis[j])
        M[i, j] = momentum(basis[i], basis[j])
    end

    vals, vecs = eigen(Hermitian(N))
    M = (vecs' * M * vecs)
    M /= diagm(vals)

    vals, vecs = eigen(M)
    return angle.(vals)
end

momenta = Float64[]
append!(momenta, fix_degeneracies(states[1:1]))
append!(momenta, fix_degeneracies(states[2:2]))
append!(momenta, fix_degeneracies(states[3:3]))
append!(momenta, fix_degeneracies(states[4:5]))
append!(momenta, fix_degeneracies(states[6:9]))
append!(momenta, fix_degeneracies(states[10:11]))
append!(momenta, fix_degeneracies(states[12:12]))
append!(momenta, fix_degeneracies(states[13:16]))
append!(momenta, fix_degeneracies(states[17:18]))

plot(momenta, real.(energies[1:18]), seriestype=:scatter, xlabel="momentum", ylabel="energy", legend=false)
````

```@raw html
<?xml version="1.0" encoding="utf-8"?>
<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="600" height="400" viewBox="0 0 2400 1600">
<defs>
  <clipPath id="clip280">
    <rect x="0" y="0" width="2400" height="1600"/>
  </clipPath>
</defs>
<path clip-path="url(#clip280)" d="M0 1600 L2400 1600 L2400 0 L0 0  Z" fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip281">
    <rect x="480" y="0" width="1681" height="1600"/>
  </clipPath>
</defs>
<path clip-path="url(#clip280)" d="M287.366 1423.18 L2352.76 1423.18 L2352.76 47.2441 L287.366 47.2441  Z" fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip282">
    <rect x="287" y="47" width="2066" height="1377"/>
  </clipPath>
</defs>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="389.73,1423.18 389.73,47.2441 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="699.84,1423.18 699.84,47.2441 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1009.95,1423.18 1009.95,47.2441 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1320.06,1423.18 1320.06,47.2441 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1630.17,1423.18 1630.17,47.2441 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1940.28,1423.18 1940.28,47.2441 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="2250.39,1423.18 2250.39,47.2441 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="287.366,1423.18 2352.76,1423.18 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="389.73,1423.18 389.73,1404.28 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="699.84,1423.18 699.84,1404.28 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1009.95,1423.18 1009.95,1404.28 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1320.06,1423.18 1320.06,1404.28 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1630.17,1423.18 1630.17,1404.28 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1940.28,1423.18 1940.28,1404.28 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="2250.39,1423.18 2250.39,1404.28 "/>
<path clip-path="url(#clip280)" d="M359.197 1468.75 L388.873 1468.75 L388.873 1472.69 L359.197 1472.69 L359.197 1468.75 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M413.132 1466.95 Q416.489 1467.66 418.364 1469.93 Q420.262 1472.2 420.262 1475.53 Q420.262 1480.65 416.743 1483.45 Q413.225 1486.25 406.743 1486.25 Q404.568 1486.25 402.253 1485.81 Q399.961 1485.39 397.507 1484.54 L397.507 1480.02 Q399.452 1481.16 401.767 1481.74 Q404.081 1482.32 406.605 1482.32 Q411.003 1482.32 413.294 1480.58 Q415.609 1478.84 415.609 1475.53 Q415.609 1472.48 413.456 1470.77 Q411.327 1469.03 407.507 1469.03 L403.48 1469.03 L403.48 1465.19 L407.693 1465.19 Q411.142 1465.19 412.97 1463.82 Q414.799 1462.43 414.799 1459.84 Q414.799 1457.18 412.901 1455.77 Q411.026 1454.33 407.507 1454.33 Q405.586 1454.33 403.387 1454.75 Q401.188 1455.16 398.549 1456.04 L398.549 1451.88 Q401.211 1451.14 403.526 1450.77 Q405.864 1450.39 407.924 1450.39 Q413.248 1450.39 416.35 1452.83 Q419.452 1455.23 419.452 1459.35 Q419.452 1462.22 417.808 1464.21 Q416.165 1466.18 413.132 1466.95 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M669.782 1468.75 L699.458 1468.75 L699.458 1472.69 L669.782 1472.69 L669.782 1468.75 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M713.578 1481.64 L729.898 1481.64 L729.898 1485.58 L707.953 1485.58 L707.953 1481.64 Q710.616 1478.89 715.199 1474.26 Q719.805 1469.61 720.986 1468.27 Q723.231 1465.74 724.111 1464.01 Q725.014 1462.25 725.014 1460.56 Q725.014 1457.8 723.069 1456.07 Q721.148 1454.33 718.046 1454.33 Q715.847 1454.33 713.393 1455.09 Q710.963 1455.86 708.185 1457.41 L708.185 1452.69 Q711.009 1451.55 713.463 1450.97 Q715.916 1450.39 717.953 1450.39 Q723.324 1450.39 726.518 1453.08 Q729.713 1455.77 729.713 1460.26 Q729.713 1462.39 728.902 1464.31 Q728.115 1466.2 726.009 1468.8 Q725.43 1469.47 722.328 1472.69 Q719.227 1475.88 713.578 1481.64 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M979.708 1468.75 L1009.38 1468.75 L1009.38 1472.69 L979.708 1472.69 L979.708 1468.75 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1020.29 1481.64 L1027.92 1481.64 L1027.92 1455.28 L1019.61 1456.95 L1019.61 1452.69 L1027.88 1451.02 L1032.55 1451.02 L1032.55 1481.64 L1040.19 1481.64 L1040.19 1485.58 L1020.29 1485.58 L1020.29 1481.64 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1320.06 1454.1 Q1316.45 1454.1 1314.62 1457.66 Q1312.82 1461.2 1312.82 1468.33 Q1312.82 1475.44 1314.62 1479.01 Q1316.45 1482.55 1320.06 1482.55 Q1323.7 1482.55 1325.5 1479.01 Q1327.33 1475.44 1327.33 1468.33 Q1327.33 1461.2 1325.5 1457.66 Q1323.7 1454.1 1320.06 1454.1 M1320.06 1450.39 Q1325.87 1450.39 1328.93 1455 Q1332.01 1459.58 1332.01 1468.33 Q1332.01 1477.06 1328.93 1481.67 Q1325.87 1486.25 1320.06 1486.25 Q1314.25 1486.25 1311.17 1481.67 Q1308.12 1477.06 1308.12 1468.33 Q1308.12 1459.58 1311.17 1455 Q1314.25 1450.39 1320.06 1450.39 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1620.55 1481.64 L1628.19 1481.64 L1628.19 1455.28 L1619.88 1456.95 L1619.88 1452.69 L1628.15 1451.02 L1632.82 1451.02 L1632.82 1481.64 L1640.46 1481.64 L1640.46 1485.58 L1620.55 1485.58 L1620.55 1481.64 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1934.93 1481.64 L1951.25 1481.64 L1951.25 1485.58 L1929.31 1485.58 L1929.31 1481.64 Q1931.97 1478.89 1936.55 1474.26 Q1941.16 1469.61 1942.34 1468.27 Q1944.59 1465.74 1945.47 1464.01 Q1946.37 1462.25 1946.37 1460.56 Q1946.37 1457.8 1944.43 1456.07 Q1942.5 1454.33 1939.4 1454.33 Q1937.2 1454.33 1934.75 1455.09 Q1932.32 1455.86 1929.54 1457.41 L1929.54 1452.69 Q1932.37 1451.55 1934.82 1450.97 Q1937.27 1450.39 1939.31 1450.39 Q1944.68 1450.39 1947.87 1453.08 Q1951.07 1455.77 1951.07 1460.26 Q1951.07 1462.39 1950.26 1464.31 Q1949.47 1466.2 1947.37 1468.8 Q1946.79 1469.47 1943.68 1472.69 Q1940.58 1475.88 1934.93 1481.64 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M2254.64 1466.95 Q2258 1467.66 2259.87 1469.93 Q2261.77 1472.2 2261.77 1475.53 Q2261.77 1480.65 2258.25 1483.45 Q2254.73 1486.25 2248.25 1486.25 Q2246.08 1486.25 2243.76 1485.81 Q2241.47 1485.39 2239.01 1484.54 L2239.01 1480.02 Q2240.96 1481.16 2243.27 1481.74 Q2245.59 1482.32 2248.11 1482.32 Q2252.51 1482.32 2254.8 1480.58 Q2257.12 1478.84 2257.12 1475.53 Q2257.12 1472.48 2254.96 1470.77 Q2252.83 1469.03 2249.01 1469.03 L2244.99 1469.03 L2244.99 1465.19 L2249.2 1465.19 Q2252.65 1465.19 2254.48 1463.82 Q2256.31 1462.43 2256.31 1459.84 Q2256.31 1457.18 2254.41 1455.77 Q2252.53 1454.33 2249.01 1454.33 Q2247.09 1454.33 2244.89 1454.75 Q2242.7 1455.16 2240.06 1456.04 L2240.06 1451.88 Q2242.72 1451.14 2245.03 1450.77 Q2247.37 1450.39 2249.43 1450.39 Q2254.76 1450.39 2257.86 1452.83 Q2260.96 1455.23 2260.96 1459.35 Q2260.96 1462.22 2259.32 1464.21 Q2257.67 1466.18 2254.64 1466.95 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1164.44 1539.24 Q1166.63 1535.29 1169.69 1533.41 Q1172.74 1531.54 1176.88 1531.54 Q1182.45 1531.54 1185.47 1535.45 Q1188.5 1539.33 1188.5 1546.53 L1188.5 1568.04 L1182.61 1568.04 L1182.61 1546.72 Q1182.61 1541.59 1180.8 1539.11 Q1178.98 1536.63 1175.26 1536.63 Q1170.71 1536.63 1168.06 1539.65 Q1165.42 1542.68 1165.42 1547.9 L1165.42 1568.04 L1159.53 1568.04 L1159.53 1546.72 Q1159.53 1541.56 1157.72 1539.11 Q1155.91 1536.63 1152.12 1536.63 Q1147.63 1536.63 1144.99 1539.68 Q1142.35 1542.71 1142.35 1547.9 L1142.35 1568.04 L1136.46 1568.04 L1136.46 1532.4 L1142.35 1532.4 L1142.35 1537.93 Q1144.35 1534.66 1147.15 1533.1 Q1149.95 1531.54 1153.8 1531.54 Q1157.69 1531.54 1160.39 1533.51 Q1163.13 1535.48 1164.44 1539.24 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1213.99 1536.5 Q1209.28 1536.5 1206.54 1540.19 Q1203.81 1543.85 1203.81 1550.25 Q1203.81 1556.65 1206.51 1560.34 Q1209.25 1564 1213.99 1564 Q1218.67 1564 1221.41 1560.31 Q1224.15 1556.62 1224.15 1550.25 Q1224.15 1543.92 1221.41 1540.23 Q1218.67 1536.5 1213.99 1536.5 M1213.99 1531.54 Q1221.63 1531.54 1225.99 1536.5 Q1230.35 1541.47 1230.35 1550.25 Q1230.35 1559 1225.99 1564 Q1221.63 1568.97 1213.99 1568.97 Q1206.32 1568.97 1201.96 1564 Q1197.63 1559 1197.63 1550.25 Q1197.63 1541.47 1201.96 1536.5 Q1206.32 1531.54 1213.99 1531.54 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1267.81 1539.24 Q1270.01 1535.29 1273.07 1533.41 Q1276.12 1531.54 1280.26 1531.54 Q1285.83 1531.54 1288.85 1535.45 Q1291.88 1539.33 1291.88 1546.53 L1291.88 1568.04 L1285.99 1568.04 L1285.99 1546.72 Q1285.99 1541.59 1284.17 1539.11 Q1282.36 1536.63 1278.64 1536.63 Q1274.08 1536.63 1271.44 1539.65 Q1268.8 1542.68 1268.8 1547.9 L1268.8 1568.04 L1262.91 1568.04 L1262.91 1546.72 Q1262.91 1541.56 1261.1 1539.11 Q1259.28 1536.63 1255.5 1536.63 Q1251.01 1536.63 1248.37 1539.68 Q1245.73 1542.71 1245.73 1547.9 L1245.73 1568.04 L1239.84 1568.04 L1239.84 1532.4 L1245.73 1532.4 L1245.73 1537.93 Q1247.73 1534.66 1250.53 1533.1 Q1253.33 1531.54 1257.18 1531.54 Q1261.07 1531.54 1263.77 1533.51 Q1266.51 1535.48 1267.81 1539.24 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1334.05 1548.76 L1334.05 1551.62 L1307.12 1551.62 Q1307.5 1557.67 1310.75 1560.85 Q1314.03 1564 1319.85 1564 Q1323.23 1564 1326.38 1563.17 Q1329.56 1562.35 1332.68 1560.69 L1332.68 1566.23 Q1329.53 1567.57 1326.22 1568.27 Q1322.91 1568.97 1319.5 1568.97 Q1310.97 1568.97 1305.98 1564 Q1301.01 1559.04 1301.01 1550.57 Q1301.01 1541.82 1305.72 1536.69 Q1310.46 1531.54 1318.49 1531.54 Q1325.68 1531.54 1329.85 1536.18 Q1334.05 1540.8 1334.05 1548.76 M1328.19 1547.04 Q1328.13 1542.23 1325.49 1539.37 Q1322.88 1536.5 1318.55 1536.5 Q1313.65 1536.5 1310.69 1539.27 Q1307.76 1542.04 1307.31 1547.07 L1328.19 1547.04 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1373.29 1546.53 L1373.29 1568.04 L1367.44 1568.04 L1367.44 1546.72 Q1367.44 1541.66 1365.46 1539.14 Q1363.49 1536.63 1359.54 1536.63 Q1354.8 1536.63 1352.06 1539.65 Q1349.33 1542.68 1349.33 1547.9 L1349.33 1568.04 L1343.44 1568.04 L1343.44 1532.4 L1349.33 1532.4 L1349.33 1537.93 Q1351.43 1534.72 1354.26 1533.13 Q1357.13 1531.54 1360.85 1531.54 Q1366.99 1531.54 1370.14 1535.36 Q1373.29 1539.14 1373.29 1546.53 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1390.77 1522.27 L1390.77 1532.4 L1402.83 1532.4 L1402.83 1536.95 L1390.77 1536.95 L1390.77 1556.3 Q1390.77 1560.66 1391.95 1561.9 Q1393.16 1563.14 1396.82 1563.14 L1402.83 1563.14 L1402.83 1568.04 L1396.82 1568.04 Q1390.04 1568.04 1387.46 1565.53 Q1384.88 1562.98 1384.88 1556.3 L1384.88 1536.95 L1380.58 1536.95 L1380.58 1532.4 L1384.88 1532.4 L1384.88 1522.27 L1390.77 1522.27 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1409.93 1553.98 L1409.93 1532.4 L1415.79 1532.4 L1415.79 1553.75 Q1415.79 1558.81 1417.76 1561.36 Q1419.73 1563.87 1423.68 1563.87 Q1428.42 1563.87 1431.16 1560.85 Q1433.93 1557.83 1433.93 1552.61 L1433.93 1532.4 L1439.78 1532.4 L1439.78 1568.04 L1433.93 1568.04 L1433.93 1562.57 Q1431.79 1565.82 1428.96 1567.41 Q1426.16 1568.97 1422.44 1568.97 Q1416.29 1568.97 1413.11 1565.15 Q1409.93 1561.33 1409.93 1553.98 M1424.67 1531.54 L1424.67 1531.54 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M1479.6 1539.24 Q1481.8 1535.29 1484.85 1533.41 Q1487.91 1531.54 1492.05 1531.54 Q1497.62 1531.54 1500.64 1535.45 Q1503.66 1539.33 1503.66 1546.53 L1503.66 1568.04 L1497.78 1568.04 L1497.78 1546.72 Q1497.78 1541.59 1495.96 1539.11 Q1494.15 1536.63 1490.42 1536.63 Q1485.87 1536.63 1483.23 1539.65 Q1480.59 1542.68 1480.59 1547.9 L1480.59 1568.04 L1474.7 1568.04 L1474.7 1546.72 Q1474.7 1541.56 1472.89 1539.11 Q1471.07 1536.63 1467.28 1536.63 Q1462.8 1536.63 1460.15 1539.68 Q1457.51 1542.71 1457.51 1547.9 L1457.51 1568.04 L1451.62 1568.04 L1451.62 1532.4 L1457.51 1532.4 L1457.51 1537.93 Q1459.52 1534.66 1462.32 1533.1 Q1465.12 1531.54 1468.97 1531.54 Q1472.85 1531.54 1475.56 1533.51 Q1478.3 1535.48 1479.6 1539.24 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="287.366,1242.75 2352.76,1242.75 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="287.366,1023.45 2352.76,1023.45 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="287.366,804.144 2352.76,804.144 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="287.366,584.842 2352.76,584.842 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="287.366,365.54 2352.76,365.54 "/>
<polyline clip-path="url(#clip282)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="287.366,146.239 2352.76,146.239 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="287.366,1423.18 287.366,47.2441 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="287.366,1242.75 306.264,1242.75 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="287.366,1023.45 306.264,1023.45 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="287.366,804.144 306.264,804.144 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="287.366,584.842 306.264,584.842 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="287.366,365.54 306.264,365.54 "/>
<polyline clip-path="url(#clip280)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="287.366,146.239 306.264,146.239 "/>
<path clip-path="url(#clip280)" d="M114.26 1243.2 L143.936 1243.2 L143.936 1247.13 L114.26 1247.13 L114.26 1243.2 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M154.839 1256.09 L162.477 1256.09 L162.477 1229.73 L154.167 1231.39 L154.167 1227.13 L162.431 1225.47 L167.107 1225.47 L167.107 1256.09 L174.746 1256.09 L174.746 1260.03 L154.839 1260.03 L154.839 1256.09 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M184.237 1225.47 L202.593 1225.47 L202.593 1229.4 L188.519 1229.4 L188.519 1237.87 Q189.538 1237.53 190.556 1237.37 Q191.575 1237.18 192.593 1237.18 Q198.38 1237.18 201.76 1240.35 Q205.139 1243.52 205.139 1248.94 Q205.139 1254.52 201.667 1257.62 Q198.195 1260.7 191.875 1260.7 Q189.7 1260.7 187.431 1260.33 Q185.186 1259.96 182.778 1259.22 L182.778 1254.52 Q184.862 1255.65 187.084 1256.21 Q189.306 1256.76 191.783 1256.76 Q195.787 1256.76 198.125 1254.66 Q200.463 1252.55 200.463 1248.94 Q200.463 1245.33 198.125 1243.22 Q195.787 1241.12 191.783 1241.12 Q189.908 1241.12 188.033 1241.53 Q186.181 1241.95 184.237 1242.83 L184.237 1225.47 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M214.352 1254.15 L219.236 1254.15 L219.236 1260.03 L214.352 1260.03 L214.352 1254.15 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M239.422 1228.55 Q235.81 1228.55 233.982 1232.11 Q232.176 1235.65 232.176 1242.78 Q232.176 1249.89 233.982 1253.45 Q235.81 1256.99 239.422 1256.99 Q243.056 1256.99 244.861 1253.45 Q246.69 1249.89 246.69 1242.78 Q246.69 1235.65 244.861 1232.11 Q243.056 1228.55 239.422 1228.55 M239.422 1224.84 Q245.232 1224.84 248.287 1229.45 Q251.366 1234.03 251.366 1242.78 Q251.366 1251.51 248.287 1256.11 Q245.232 1260.7 239.422 1260.7 Q233.611 1260.7 230.533 1256.11 Q227.477 1251.51 227.477 1242.78 Q227.477 1234.03 230.533 1229.45 Q233.611 1224.84 239.422 1224.84 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M115.256 1023.9 L144.931 1023.9 L144.931 1027.83 L115.256 1027.83 L115.256 1023.9 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M155.834 1036.79 L163.473 1036.79 L163.473 1010.42 L155.163 1012.09 L155.163 1007.83 L163.427 1006.17 L168.102 1006.17 L168.102 1036.79 L175.741 1036.79 L175.741 1040.73 L155.834 1040.73 L155.834 1036.79 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M198.033 1010.24 L186.227 1028.69 L198.033 1028.69 L198.033 1010.24 M196.806 1006.17 L202.686 1006.17 L202.686 1028.69 L207.616 1028.69 L207.616 1032.58 L202.686 1032.58 L202.686 1040.73 L198.033 1040.73 L198.033 1032.58 L182.431 1032.58 L182.431 1028.06 L196.806 1006.17 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M215.348 1034.85 L220.232 1034.85 L220.232 1040.73 L215.348 1040.73 L215.348 1034.85 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M230.463 1006.17 L248.82 1006.17 L248.82 1010.1 L234.746 1010.1 L234.746 1018.57 Q235.764 1018.23 236.783 1018.06 Q237.801 1017.88 238.82 1017.88 Q244.607 1017.88 247.986 1021.05 Q251.366 1024.22 251.366 1029.64 Q251.366 1035.22 247.894 1038.32 Q244.421 1041.4 238.102 1041.4 Q235.926 1041.4 233.658 1041.03 Q231.412 1040.66 229.005 1039.92 L229.005 1035.22 Q231.088 1036.35 233.31 1036.91 Q235.533 1037.46 238.009 1037.46 Q242.014 1037.46 244.352 1035.35 Q246.69 1033.25 246.69 1029.64 Q246.69 1026.03 244.352 1023.92 Q242.014 1021.81 238.009 1021.81 Q236.135 1021.81 234.26 1022.23 Q232.408 1022.65 230.463 1023.53 L230.463 1006.17 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M114.26 804.595 L143.936 804.595 L143.936 808.53 L114.26 808.53 L114.26 804.595 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M154.839 817.489 L162.477 817.489 L162.477 791.123 L154.167 792.79 L154.167 788.53 L162.431 786.864 L167.107 786.864 L167.107 817.489 L174.746 817.489 L174.746 821.424 L154.839 821.424 L154.839 817.489 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M197.037 790.938 L185.232 809.387 L197.037 809.387 L197.037 790.938 M195.811 786.864 L201.69 786.864 L201.69 809.387 L206.621 809.387 L206.621 813.276 L201.69 813.276 L201.69 821.424 L197.037 821.424 L197.037 813.276 L181.436 813.276 L181.436 808.762 L195.811 786.864 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M214.352 815.544 L219.236 815.544 L219.236 821.424 L214.352 821.424 L214.352 815.544 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M239.422 789.942 Q235.81 789.942 233.982 793.507 Q232.176 797.049 232.176 804.178 Q232.176 811.285 233.982 814.85 Q235.81 818.391 239.422 818.391 Q243.056 818.391 244.861 814.85 Q246.69 811.285 246.69 804.178 Q246.69 797.049 244.861 793.507 Q243.056 789.942 239.422 789.942 M239.422 786.239 Q245.232 786.239 248.287 790.845 Q251.366 795.428 251.366 804.178 Q251.366 812.905 248.287 817.512 Q245.232 822.095 239.422 822.095 Q233.611 822.095 230.533 817.512 Q227.477 812.905 227.477 804.178 Q227.477 795.428 230.533 790.845 Q233.611 786.239 239.422 786.239 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M115.256 585.293 L144.931 585.293 L144.931 589.229 L115.256 589.229 L115.256 585.293 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M155.834 598.187 L163.473 598.187 L163.473 571.821 L155.163 573.488 L155.163 569.229 L163.427 567.562 L168.102 567.562 L168.102 598.187 L175.741 598.187 L175.741 602.122 L155.834 602.122 L155.834 598.187 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M199.352 583.488 Q202.709 584.205 204.584 586.474 Q206.482 588.742 206.482 592.076 Q206.482 597.192 202.963 599.992 Q199.445 602.793 192.963 602.793 Q190.788 602.793 188.473 602.354 Q186.181 601.937 183.727 601.08 L183.727 596.567 Q185.672 597.701 187.987 598.279 Q190.301 598.858 192.825 598.858 Q197.223 598.858 199.514 597.122 Q201.829 595.386 201.829 592.076 Q201.829 589.02 199.676 587.307 Q197.547 585.571 193.727 585.571 L189.7 585.571 L189.7 581.729 L193.913 581.729 Q197.362 581.729 199.19 580.363 Q201.019 578.974 201.019 576.381 Q201.019 573.719 199.121 572.307 Q197.246 570.872 193.727 570.872 Q191.806 570.872 189.607 571.289 Q187.408 571.706 184.769 572.585 L184.769 568.419 Q187.431 567.678 189.746 567.307 Q192.084 566.937 194.144 566.937 Q199.468 566.937 202.57 569.368 Q205.672 571.775 205.672 575.895 Q205.672 578.766 204.028 580.756 Q202.385 582.724 199.352 583.488 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M215.348 596.242 L220.232 596.242 L220.232 602.122 L215.348 602.122 L215.348 596.242 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M230.463 567.562 L248.82 567.562 L248.82 571.497 L234.746 571.497 L234.746 579.969 Q235.764 579.622 236.783 579.46 Q237.801 579.275 238.82 579.275 Q244.607 579.275 247.986 582.446 Q251.366 585.617 251.366 591.034 Q251.366 596.613 247.894 599.715 Q244.421 602.793 238.102 602.793 Q235.926 602.793 233.658 602.423 Q231.412 602.053 229.005 601.312 L229.005 596.613 Q231.088 597.747 233.31 598.303 Q235.533 598.858 238.009 598.858 Q242.014 598.858 244.352 596.752 Q246.69 594.645 246.69 591.034 Q246.69 587.423 244.352 585.317 Q242.014 583.21 238.009 583.21 Q236.135 583.21 234.26 583.627 Q232.408 584.043 230.463 584.923 L230.463 567.562 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M114.26 365.992 L143.936 365.992 L143.936 369.927 L114.26 369.927 L114.26 365.992 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M154.839 378.885 L162.477 378.885 L162.477 352.52 L154.167 354.186 L154.167 349.927 L162.431 348.26 L167.107 348.26 L167.107 378.885 L174.746 378.885 L174.746 382.82 L154.839 382.82 L154.839 378.885 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M198.357 364.186 Q201.713 364.904 203.588 367.172 Q205.487 369.441 205.487 372.774 Q205.487 377.89 201.968 380.691 Q198.45 383.492 191.968 383.492 Q189.792 383.492 187.477 383.052 Q185.186 382.635 182.732 381.779 L182.732 377.265 Q184.676 378.399 186.991 378.978 Q189.306 379.557 191.829 379.557 Q196.227 379.557 198.519 377.82 Q200.834 376.084 200.834 372.774 Q200.834 369.719 198.681 368.006 Q196.551 366.27 192.732 366.27 L188.704 366.27 L188.704 362.427 L192.917 362.427 Q196.366 362.427 198.195 361.061 Q200.024 359.672 200.024 357.08 Q200.024 354.418 198.125 353.006 Q196.25 351.571 192.732 351.571 Q190.811 351.571 188.612 351.987 Q186.413 352.404 183.774 353.284 L183.774 349.117 Q186.436 348.376 188.75 348.006 Q191.088 347.635 193.149 347.635 Q198.473 347.635 201.575 350.066 Q204.676 352.473 204.676 356.594 Q204.676 359.464 203.033 361.455 Q201.389 363.422 198.357 364.186 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M214.352 376.941 L219.236 376.941 L219.236 382.82 L214.352 382.82 L214.352 376.941 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M239.422 351.339 Q235.81 351.339 233.982 354.904 Q232.176 358.446 232.176 365.575 Q232.176 372.682 233.982 376.246 Q235.81 379.788 239.422 379.788 Q243.056 379.788 244.861 376.246 Q246.69 372.682 246.69 365.575 Q246.69 358.446 244.861 354.904 Q243.056 351.339 239.422 351.339 M239.422 347.635 Q245.232 347.635 248.287 352.242 Q251.366 356.825 251.366 365.575 Q251.366 374.302 248.287 378.908 Q245.232 383.492 239.422 383.492 Q233.611 383.492 230.533 378.908 Q227.477 374.302 227.477 365.575 Q227.477 356.825 230.533 352.242 Q233.611 347.635 239.422 347.635 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M115.256 146.69 L144.931 146.69 L144.931 150.625 L115.256 150.625 L115.256 146.69 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M155.834 159.584 L163.473 159.584 L163.473 133.218 L155.163 134.885 L155.163 130.625 L163.427 128.959 L168.102 128.959 L168.102 159.584 L175.741 159.584 L175.741 163.519 L155.834 163.519 L155.834 159.584 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M189.213 159.584 L205.533 159.584 L205.533 163.519 L183.588 163.519 L183.588 159.584 Q186.251 156.829 190.834 152.199 Q195.44 147.547 196.621 146.204 Q198.866 143.681 199.746 141.945 Q200.649 140.186 200.649 138.496 Q200.649 135.741 198.704 134.005 Q196.783 132.269 193.681 132.269 Q191.482 132.269 189.028 133.033 Q186.598 133.797 183.82 135.348 L183.82 130.625 Q186.644 129.491 189.098 128.912 Q191.551 128.334 193.588 128.334 Q198.959 128.334 202.153 131.019 Q205.348 133.704 205.348 138.195 Q205.348 140.324 204.537 142.246 Q203.75 144.144 201.644 146.736 Q201.065 147.408 197.963 150.625 Q194.862 153.82 189.213 159.584 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M215.348 157.639 L220.232 157.639 L220.232 163.519 L215.348 163.519 L215.348 157.639 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M230.463 128.959 L248.82 128.959 L248.82 132.894 L234.746 132.894 L234.746 141.366 Q235.764 141.019 236.783 140.857 Q237.801 140.672 238.82 140.672 Q244.607 140.672 247.986 143.843 Q251.366 147.014 251.366 152.431 Q251.366 158.01 247.894 161.111 Q244.421 164.19 238.102 164.19 Q235.926 164.19 233.658 163.82 Q231.412 163.449 229.005 162.709 L229.005 158.01 Q231.088 159.144 233.31 159.699 Q235.533 160.255 238.009 160.255 Q242.014 160.255 244.352 158.148 Q246.69 156.042 246.69 152.431 Q246.69 148.82 244.352 146.713 Q242.014 144.607 238.009 144.607 Q236.135 144.607 234.26 145.023 Q232.408 145.44 230.463 146.32 L230.463 128.959 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M44.7161 812.969 L47.5806 812.969 L47.5806 839.896 Q53.6281 839.514 56.8109 836.268 Q59.9619 832.989 59.9619 827.165 Q59.9619 823.791 59.1344 820.64 Q58.3069 817.457 56.6518 814.338 L62.1899 814.338 Q63.5267 817.489 64.227 820.799 Q64.9272 824.109 64.9272 827.515 Q64.9272 836.045 59.9619 841.042 Q54.9967 846.007 46.5303 846.007 Q37.7774 846.007 32.6531 841.296 Q27.4968 836.554 27.4968 828.533 Q27.4968 821.34 32.1438 817.17 Q36.7589 812.969 44.7161 812.969 M42.9973 818.826 Q38.1912 818.889 35.3266 821.531 Q32.4621 824.141 32.4621 828.47 Q32.4621 833.371 35.2312 836.331 Q38.0002 839.259 43.0292 839.705 L42.9973 818.826 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M42.4881 773.724 L64.0042 773.724 L64.0042 779.581 L42.679 779.581 Q37.6183 779.581 35.1038 781.554 Q32.5894 783.528 32.5894 787.474 Q32.5894 792.217 35.6131 794.954 Q38.6368 797.691 43.8567 797.691 L64.0042 797.691 L64.0042 803.58 L28.3562 803.58 L28.3562 797.691 L33.8944 797.691 Q30.6797 795.591 29.0883 792.758 Q27.4968 789.893 27.4968 786.169 Q27.4968 780.027 31.3163 776.876 Q35.1038 773.724 42.4881 773.724 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M44.7161 731.552 L47.5806 731.552 L47.5806 758.479 Q53.6281 758.097 56.8109 754.85 Q59.9619 751.572 59.9619 745.747 Q59.9619 742.373 59.1344 739.222 Q58.3069 736.04 56.6518 732.92 L62.1899 732.92 Q63.5267 736.071 64.227 739.382 Q64.9272 742.692 64.9272 746.097 Q64.9272 754.627 59.9619 759.624 Q54.9967 764.59 46.5303 764.59 Q37.7774 764.59 32.6531 759.879 Q27.4968 755.137 27.4968 747.116 Q27.4968 739.923 32.1438 735.753 Q36.7589 731.552 44.7161 731.552 M42.9973 737.408 Q38.1912 737.472 35.3266 740.114 Q32.4621 742.724 32.4621 747.052 Q32.4621 751.954 35.2312 754.914 Q38.0002 757.842 43.0292 758.288 L42.9973 737.408 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M33.8307 701.283 Q33.2578 702.269 33.0032 703.447 Q32.7167 704.593 32.7167 705.993 Q32.7167 710.959 35.9632 713.632 Q39.1779 716.274 45.2253 716.274 L64.0042 716.274 L64.0042 722.162 L28.3562 722.162 L28.3562 716.274 L33.8944 716.274 Q30.6479 714.428 29.0883 711.468 Q27.4968 708.508 27.4968 704.275 Q27.4968 703.67 27.5923 702.938 Q27.656 702.206 27.8151 701.315 L33.8307 701.283 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M45.7664 672.828 Q39.4007 672.828 35.8996 675.47 Q32.3984 678.08 32.3984 682.822 Q32.3984 687.533 35.8996 690.175 Q39.4007 692.785 45.7664 692.785 Q52.1003 692.785 55.6014 690.175 Q59.1026 687.533 59.1026 682.822 Q59.1026 678.08 55.6014 675.47 Q52.1003 672.828 45.7664 672.828 M59.58 666.972 Q68.683 666.972 73.1071 671.014 Q77.5631 675.056 77.5631 683.395 Q77.5631 686.482 77.0857 689.22 Q76.6401 691.957 75.6852 694.535 L69.9879 694.535 Q71.3884 691.957 72.0568 689.443 Q72.7252 686.928 72.7252 684.318 Q72.7252 678.557 69.7015 675.693 Q66.7096 672.828 60.6303 672.828 L57.7339 672.828 Q60.885 674.642 62.4446 677.475 Q64.0042 680.308 64.0042 684.254 Q64.0042 690.811 59.0071 694.822 Q54.01 698.832 45.7664 698.832 Q37.491 698.832 32.4939 694.822 Q27.4968 690.811 27.4968 684.254 Q27.4968 680.308 29.0564 677.475 Q30.616 674.642 33.7671 672.828 L28.3562 672.828 L28.3562 666.972 L59.58 666.972 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip280)" d="M67.3143 640.076 Q73.68 642.559 75.6216 644.914 Q77.5631 647.27 77.5631 651.216 L77.5631 655.895 L72.6615 655.895 L72.6615 652.458 Q72.6615 650.039 71.5157 648.702 Q70.3699 647.365 66.1048 645.742 L63.4312 644.692 L28.3562 659.11 L28.3562 652.903 L56.238 641.763 L28.3562 630.623 L28.3562 624.417 L67.3143 640.076 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><circle clip-path="url(#clip282)" cx="1320.06" cy="1384.24" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="2294.3" cy="1326.74" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="1320.06" cy="926.245" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="2294.3" cy="872.668" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="1320.06" cy="872.668" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="2294.3" cy="483.857" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="2294.3" cy="483.857" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="1150.58" cy="483.857" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="1489.55" cy="483.857" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="2294.3" cy="449.537" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="1320.06" cy="449.537" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="2294.3" cy="418.592" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="520.515" cy="87.2223" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="2119.61" cy="87.2223" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="1367.25" cy="87.2223" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="1272.87" cy="87.2223" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="345.82" cy="86.1857" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip282)" cx="1320.06" cy="86.1857" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
</svg>

```

## Finite bond dimension

If we limit the maximum bond dimension of the MPS, we get an approximate solution, but we
can reach higher system sizes.

````julia
L_mps = 20
H_mps = periodic_boundary_conditions(transverse_field_ising(), L_mps)
D = 64
ψ, envs, δ = find_groundstate(FiniteMPS(L_mps, ℂ^2, ℂ^D), H_mps, DMRG());
````

````
┌ Warning: failed to deduce all domspaces
└ @ MPSKit ~/projects/MPSKit.jl/src/operators/sparsempo/sparsempo.jl:144
[ Info: Iteraton 0 error 0.027472963540396158
[ Info: Iteraton 1 error 4.2499990746454e-6
[ Info: Iteraton 2 error 4.518939647996781e-7
[ Info: Iteraton 3 error 1.4266935739038215e-7
[ Info: Iteraton 4 error 5.812790872635475e-8
[ Info: Iteraton 5 error 3.393604922259721e-8
[ Info: Iteraton 6 error 3.508323755418118e-8
[ Info: Iteraton 7 error 4.318268478143819e-8
[ Info: Iteraton 8 error 5.122181044220508e-8
[ Info: Iteraton 9 error 5.7292768222888516e-8
[ Info: Iteraton 10 error 5.928908338653149e-8
[ Info: Iteraton 11 error 5.792982675326755e-8
[ Info: Iteraton 12 error 5.251579077114723e-8
[ Info: Iteraton 13 error 4.416508660099376e-8
[ Info: Iteraton 14 error 3.5163073134686605e-8
[ Info: Iteraton 15 error 2.7034964301287772e-8
[ Info: Iteraton 16 error 2.0374010758048726e-8
[ Info: Iteraton 17 error 1.519740629741549e-8
[ Info: Iteraton 18 error 1.1287063199184159e-8
[ Info: Iteraton 19 error 8.376252613910147e-9
[ Info: Iteraton 20 error 6.224642265905982e-9
[ Info: Iteraton 21 error 4.638535397803285e-9
[ Info: Iteraton 22 error 3.4695667576124297e-9
[ Info: Iteraton 23 error 2.6069109293991193e-9
[ Info: Iteraton 24 error 1.968826919112749e-9
[ Info: Iteraton 25 error 1.4954022020985557e-9
[ Info: Iteraton 26 error 1.1428476693160124e-9
[ Info: Iteraton 27 error 1.0052245724187655e-9
[ Info: Iteraton 28 error 9.376197681400316e-10
[ Info: Iteraton 29 error 9.003965381666546e-10
[ Info: Iteraton 30 error 8.650372631298306e-10
[ Info: Iteraton 31 error 8.313368894489376e-10
[ Info: Iteraton 32 error 7.991329391813984e-10
[ Info: Iteraton 33 error 7.682945984183869e-10
[ Info: Iteraton 34 error 7.387158863750439e-10
[ Info: Iteraton 35 error 7.103089634350254e-10
[ Info: Iteraton 36 error 6.830003660323164e-10
[ Info: Iteraton 37 error 6.567287610228936e-10
[ Info: Iteraton 38 error 6.314408402864286e-10
[ Info: Iteraton 39 error 6.070906925446064e-10
[ Info: Iteraton 40 error 5.836368638194547e-10
[ Info: Iteraton 41 error 5.610437872178115e-10
[ Info: Iteraton 42 error 5.392778707896691e-10
[ Info: Iteraton 43 error 5.183085880417549e-10
[ Info: Iteraton 44 error 4.981076105179631e-10
[ Info: Iteraton 45 error 4.786482617538885e-10
[ Info: Iteraton 46 error 4.599055883703678e-10
[ Info: Iteraton 47 error 4.4185513735999085e-10
[ Info: Iteraton 48 error 4.2447407351739853e-10
[ Info: Iteraton 49 error 4.077401096296135e-10
[ Info: Iteraton 50 error 3.916323472504938e-10
[ Info: Iteraton 51 error 3.761297884077773e-10
[ Info: Iteraton 52 error 3.6121161753673396e-10
[ Info: Iteraton 53 error 3.468589452053742e-10
[ Info: Iteraton 54 error 3.3305227173275425e-10
[ Info: Iteraton 55 error 3.19773130530276e-10
[ Info: Iteraton 56 error 3.070033607630675e-10
[ Info: Iteraton 57 error 2.947253091408608e-10
[ Info: Iteraton 58 error 2.8292172410658897e-10
[ Info: Iteraton 59 error 2.7157598582416426e-10
[ Info: Iteraton 60 error 2.606717652380316e-10
[ Info: Iteraton 61 error 2.501931617475884e-10
[ Info: Iteraton 62 error 2.4012486547992006e-10
[ Info: Iteraton 63 error 2.3045207610776269e-10
[ Info: Iteraton 64 error 2.211598777917664e-10
[ Info: Iteraton 65 error 2.1223451912252295e-10
[ Info: Iteraton 66 error 2.0366226455767463e-10
[ Info: Iteraton 67 error 1.954297993331267e-10
[ Info: Iteraton 68 error 1.8751583829053747e-10
[ Info: Iteraton 69 error 1.799269788069608e-10
[ Info: Iteraton 70 error 1.7265239198205696e-10
[ Info: Iteraton 71 error 1.6564181622836898e-10
[ Info: Iteraton 72 error 1.589254251897116e-10
[ Info: Iteraton 73 error 1.524783138584775e-10
[ Info: Iteraton 74 error 1.4628994000080126e-10
[ Info: Iteraton 75 error 1.403492079612858e-10
[ Info: Iteraton 76 error 1.3465123802125433e-10
[ Info: Iteraton 77 error 1.291785793126184e-10
[ Info: Iteraton 78 error 1.2393031576018561e-10
[ Info: Iteraton 79 error 1.1888714150257146e-10
[ Info: Iteraton 80 error 1.1405039813461729e-10
[ Info: Iteraton 81 error 1.0941463810293189e-10
[ Info: Iteraton 82 error 1.049470772122233e-10
[ Info: Iteraton 83 error 1.0068944120957693e-10
[ Info: Iteraton 84 error 9.658084685949789e-11
[ Info: Iteraton 85 error 9.26521585213163e-11
[ Info: Iteraton 86 error 8.8873533822986e-11
[ Info: Iteraton 87 error 8.525987592024026e-11
[ Info: Iteraton 88 error 8.176699668961743e-11
[ Info: Iteraton 89 error 7.844822242784917e-11
[ Info: Iteraton 90 error 7.525441570658119e-11
[ Info: Iteraton 91 error 7.218366702838636e-11
[ Info: Iteraton 92 error 6.924429519135056e-11
[ Info: Iteraton 93 error 6.642445185586198e-11
[ Info: Iteraton 94 error 6.371187040000673e-11
[ Info: Iteraton 95 error 6.111235923883613e-11
[ Info: Iteraton 96 error 5.862208360865648e-11
[ Info: Iteraton 97 error 5.622925679053845e-11
[ Info: Iteraton 98 error 5.393398098587054e-11
[ Info: Iteraton 99 error 5.173779373536982e-11

````

Excitations on top of the groundstate can be found through the use of the quasiparticle
ansatz. This returns quasiparticle states, which can be converted to regular `FiniteMPS`
objects.

````julia
E_ex, qps = excitations(H, QuasiparticleAnsatz(), ψ, envs, num=16)
states_mps = vcat(ψ, map(qp -> convert(FiniteMPS, qp), qps))
E_mps = map(x -> sum(expectation_value(x, H_mps)), states_mps)

T_mps = periodic_boundary_conditions(DenseMPO(O), L_mps)
momenta_mps = Float64[]
append!(momenta_mps, fix_degeneracies(states[1:1]))
append!(momenta_mps, fix_degeneracies(states[2:2]))
append!(momenta_mps, fix_degeneracies(states[3:3]))
append!(momenta_mps, fix_degeneracies(states[4:5]))
append!(momenta_mps, fix_degeneracies(states[6:9]))
append!(momenta_mps, fix_degeneracies(states[10:11]))
append!(momenta_mps, fix_degeneracies(states[12:12]))
append!(momenta_mps, fix_degeneracies(states[13:16]))

plot(momenta_mps, real.(energies[1:16]), seriestype=:scatter, xlabel="momentum", ylabel="energy", legend=false)
````

```@raw html
<?xml version="1.0" encoding="utf-8"?>
<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="600" height="400" viewBox="0 0 2400 1600">
<defs>
  <clipPath id="clip310">
    <rect x="0" y="0" width="2400" height="1600"/>
  </clipPath>
</defs>
<path clip-path="url(#clip310)" d="M0 1600 L2400 1600 L2400 0 L0 0  Z" fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip311">
    <rect x="480" y="0" width="1681" height="1600"/>
  </clipPath>
</defs>
<path clip-path="url(#clip310)" d="M102.74 1505.26 L2352.76 1505.26 L2352.76 47.2441 L102.74 47.2441  Z" fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip312">
    <rect x="102" y="47" width="2251" height="1459"/>
  </clipPath>
</defs>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="381.015,1505.26 381.015,47.2441 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="752.118,1505.26 752.118,47.2441 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1123.22,1505.26 1123.22,47.2441 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1494.32,1505.26 1494.32,47.2441 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1865.43,1505.26 1865.43,47.2441 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="2236.53,1505.26 2236.53,47.2441 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="102.74,1505.26 2352.76,1505.26 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="381.015,1505.26 381.015,1486.36 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="752.118,1505.26 752.118,1486.36 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1123.22,1505.26 1123.22,1486.36 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1494.32,1505.26 1494.32,1486.36 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1865.43,1505.26 1865.43,1486.36 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="2236.53,1505.26 2236.53,1486.36 "/>
<path clip-path="url(#clip310)" d="M350.957 1550.83 L380.633 1550.83 L380.633 1554.77 L350.957 1554.77 L350.957 1550.83 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M394.753 1563.72 L411.073 1563.72 L411.073 1567.66 L389.128 1567.66 L389.128 1563.72 Q391.79 1560.97 396.374 1556.34 Q400.98 1551.69 402.161 1550.35 Q404.406 1547.82 405.286 1546.09 Q406.188 1544.33 406.188 1542.64 Q406.188 1539.88 404.244 1538.15 Q402.323 1536.41 399.221 1536.41 Q397.022 1536.41 394.568 1537.17 Q392.137 1537.94 389.36 1539.49 L389.36 1534.77 Q392.184 1533.63 394.637 1533.05 Q397.091 1532.47 399.128 1532.47 Q404.499 1532.47 407.693 1535.16 Q410.887 1537.85 410.887 1542.34 Q410.887 1544.47 410.077 1546.39 Q409.29 1548.28 407.184 1550.88 Q406.605 1551.55 403.503 1554.77 Q400.401 1557.96 394.753 1563.72 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M721.875 1550.83 L751.551 1550.83 L751.551 1554.77 L721.875 1554.77 L721.875 1550.83 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M762.454 1563.72 L770.092 1563.72 L770.092 1537.36 L761.782 1539.03 L761.782 1534.77 L770.046 1533.1 L774.722 1533.1 L774.722 1563.72 L782.361 1563.72 L782.361 1567.66 L762.454 1567.66 L762.454 1563.72 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1123.22 1536.18 Q1119.61 1536.18 1117.78 1539.74 Q1115.98 1543.28 1115.98 1550.41 Q1115.98 1557.52 1117.78 1561.09 Q1119.61 1564.63 1123.22 1564.63 Q1126.86 1564.63 1128.66 1561.09 Q1130.49 1557.52 1130.49 1550.41 Q1130.49 1543.28 1128.66 1539.74 Q1126.86 1536.18 1123.22 1536.18 M1123.22 1532.47 Q1129.03 1532.47 1132.09 1537.08 Q1135.17 1541.66 1135.17 1550.41 Q1135.17 1559.14 1132.09 1563.75 Q1129.03 1568.33 1123.22 1568.33 Q1117.41 1568.33 1114.33 1563.75 Q1111.28 1559.14 1111.28 1550.41 Q1111.28 1541.66 1114.33 1537.08 Q1117.41 1532.47 1123.22 1532.47 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1484.71 1563.72 L1492.35 1563.72 L1492.35 1537.36 L1484.04 1539.03 L1484.04 1534.77 L1492.3 1533.1 L1496.97 1533.1 L1496.97 1563.72 L1504.61 1563.72 L1504.61 1567.66 L1484.71 1567.66 L1484.71 1563.72 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1860.08 1563.72 L1876.4 1563.72 L1876.4 1567.66 L1854.46 1567.66 L1854.46 1563.72 Q1857.12 1560.97 1861.7 1556.34 Q1866.31 1551.69 1867.49 1550.35 Q1869.73 1547.82 1870.61 1546.09 Q1871.52 1544.33 1871.52 1542.64 Q1871.52 1539.88 1869.57 1538.15 Q1867.65 1536.41 1864.55 1536.41 Q1862.35 1536.41 1859.9 1537.17 Q1857.46 1537.94 1854.69 1539.49 L1854.69 1534.77 Q1857.51 1533.63 1859.96 1533.05 Q1862.42 1532.47 1864.46 1532.47 Q1869.83 1532.47 1873.02 1535.16 Q1876.21 1537.85 1876.21 1542.34 Q1876.21 1544.47 1875.4 1546.39 Q1874.62 1548.28 1872.51 1550.88 Q1871.93 1551.55 1868.83 1554.77 Q1865.73 1557.96 1860.08 1563.72 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M2240.78 1549.03 Q2244.13 1549.74 2246.01 1552.01 Q2247.91 1554.28 2247.91 1557.61 Q2247.91 1562.73 2244.39 1565.53 Q2240.87 1568.33 2234.39 1568.33 Q2232.21 1568.33 2229.9 1567.89 Q2227.61 1567.47 2225.15 1566.62 L2225.15 1562.1 Q2227.1 1563.24 2229.41 1563.82 Q2231.73 1564.4 2234.25 1564.4 Q2238.65 1564.4 2240.94 1562.66 Q2243.26 1560.92 2243.26 1557.61 Q2243.26 1554.56 2241.1 1552.85 Q2238.97 1551.11 2235.15 1551.11 L2231.13 1551.11 L2231.13 1547.27 L2235.34 1547.27 Q2238.79 1547.27 2240.62 1545.9 Q2242.45 1544.51 2242.45 1541.92 Q2242.45 1539.26 2240.55 1537.85 Q2238.67 1536.41 2235.15 1536.41 Q2233.23 1536.41 2231.03 1536.83 Q2228.83 1537.24 2226.2 1538.12 L2226.2 1533.96 Q2228.86 1533.22 2231.17 1532.85 Q2233.51 1532.47 2235.57 1532.47 Q2240.89 1532.47 2244 1534.91 Q2247.1 1537.31 2247.1 1541.43 Q2247.1 1544.3 2245.45 1546.29 Q2243.81 1548.26 2240.78 1549.03 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1072.12 1621.32 Q1074.32 1617.37 1077.37 1615.49 Q1080.43 1613.62 1084.57 1613.62 Q1090.14 1613.62 1093.16 1617.53 Q1096.18 1621.41 1096.18 1628.61 L1096.18 1650.12 L1090.3 1650.12 L1090.3 1628.8 Q1090.3 1623.67 1088.48 1621.19 Q1086.67 1618.71 1082.94 1618.71 Q1078.39 1618.71 1075.75 1621.73 Q1073.11 1624.76 1073.11 1629.98 L1073.11 1650.12 L1067.22 1650.12 L1067.22 1628.8 Q1067.22 1623.64 1065.41 1621.19 Q1063.59 1618.71 1059.8 1618.71 Q1055.32 1618.71 1052.68 1621.76 Q1050.03 1624.79 1050.03 1629.98 L1050.03 1650.12 L1044.15 1650.12 L1044.15 1614.48 L1050.03 1614.48 L1050.03 1620.01 Q1052.04 1616.74 1054.84 1615.18 Q1057.64 1613.62 1061.49 1613.62 Q1065.37 1613.62 1068.08 1615.59 Q1070.82 1617.56 1072.12 1621.32 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1121.68 1618.58 Q1116.97 1618.58 1114.23 1622.27 Q1111.49 1625.93 1111.49 1632.33 Q1111.49 1638.73 1114.2 1642.42 Q1116.94 1646.08 1121.68 1646.08 Q1126.36 1646.08 1129.1 1642.39 Q1131.83 1638.7 1131.83 1632.33 Q1131.83 1626 1129.1 1622.31 Q1126.36 1618.58 1121.68 1618.58 M1121.68 1613.62 Q1129.32 1613.62 1133.68 1618.58 Q1138.04 1623.55 1138.04 1632.33 Q1138.04 1641.08 1133.68 1646.08 Q1129.32 1651.05 1121.68 1651.05 Q1114.01 1651.05 1109.65 1646.08 Q1105.32 1641.08 1105.32 1632.33 Q1105.32 1623.55 1109.65 1618.58 Q1114.01 1613.62 1121.68 1613.62 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1175.5 1621.32 Q1177.7 1617.37 1180.75 1615.49 Q1183.81 1613.62 1187.95 1613.62 Q1193.52 1613.62 1196.54 1617.53 Q1199.56 1621.41 1199.56 1628.61 L1199.56 1650.12 L1193.68 1650.12 L1193.68 1628.8 Q1193.68 1623.67 1191.86 1621.19 Q1190.05 1618.71 1186.32 1618.71 Q1181.77 1618.71 1179.13 1621.73 Q1176.49 1624.76 1176.49 1629.98 L1176.49 1650.12 L1170.6 1650.12 L1170.6 1628.8 Q1170.6 1623.64 1168.79 1621.19 Q1166.97 1618.71 1163.18 1618.71 Q1158.7 1618.71 1156.05 1621.76 Q1153.41 1624.79 1153.41 1629.98 L1153.41 1650.12 L1147.52 1650.12 L1147.52 1614.48 L1153.41 1614.48 L1153.41 1620.01 Q1155.42 1616.74 1158.22 1615.18 Q1161.02 1613.62 1164.87 1613.62 Q1168.75 1613.62 1171.46 1615.59 Q1174.2 1617.56 1175.5 1621.32 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1241.74 1630.84 L1241.74 1633.7 L1214.81 1633.7 Q1215.19 1639.75 1218.44 1642.93 Q1221.72 1646.08 1227.54 1646.08 Q1230.91 1646.08 1234.07 1645.25 Q1237.25 1644.43 1240.37 1642.77 L1240.37 1648.31 Q1237.22 1649.65 1233.91 1650.35 Q1230.6 1651.05 1227.19 1651.05 Q1218.66 1651.05 1213.66 1646.08 Q1208.7 1641.12 1208.7 1632.65 Q1208.7 1623.9 1213.41 1618.77 Q1218.15 1613.62 1226.17 1613.62 Q1233.37 1613.62 1237.54 1618.26 Q1241.74 1622.88 1241.74 1630.84 M1235.88 1629.12 Q1235.82 1624.31 1233.17 1621.45 Q1230.56 1618.58 1226.24 1618.58 Q1221.33 1618.58 1218.37 1621.35 Q1215.45 1624.12 1215 1629.15 L1235.88 1629.12 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1280.98 1628.61 L1280.98 1650.12 L1275.12 1650.12 L1275.12 1628.8 Q1275.12 1623.74 1273.15 1621.22 Q1271.18 1618.71 1267.23 1618.71 Q1262.49 1618.71 1259.75 1621.73 Q1257.01 1624.76 1257.01 1629.98 L1257.01 1650.12 L1251.13 1650.12 L1251.13 1614.48 L1257.01 1614.48 L1257.01 1620.01 Q1259.12 1616.8 1261.95 1615.21 Q1264.81 1613.62 1268.54 1613.62 Q1274.68 1613.62 1277.83 1617.44 Q1280.98 1621.22 1280.98 1628.61 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1298.46 1604.35 L1298.46 1614.48 L1310.52 1614.48 L1310.52 1619.03 L1298.46 1619.03 L1298.46 1638.38 Q1298.46 1642.74 1299.63 1643.98 Q1300.84 1645.22 1304.5 1645.22 L1310.52 1645.22 L1310.52 1650.12 L1304.5 1650.12 Q1297.72 1650.12 1295.14 1647.61 Q1292.57 1645.06 1292.57 1638.38 L1292.57 1619.03 L1288.27 1619.03 L1288.27 1614.48 L1292.57 1614.48 L1292.57 1604.35 L1298.46 1604.35 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1317.62 1636.06 L1317.62 1614.48 L1323.47 1614.48 L1323.47 1635.83 Q1323.47 1640.89 1325.45 1643.44 Q1327.42 1645.95 1331.37 1645.95 Q1336.11 1645.95 1338.85 1642.93 Q1341.61 1639.91 1341.61 1634.69 L1341.61 1614.48 L1347.47 1614.48 L1347.47 1650.12 L1341.61 1650.12 L1341.61 1644.65 Q1339.48 1647.9 1336.65 1649.49 Q1333.85 1651.05 1330.12 1651.05 Q1323.98 1651.05 1320.8 1647.23 Q1317.62 1643.41 1317.62 1636.06 M1332.35 1613.62 L1332.35 1613.62 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M1387.29 1621.32 Q1389.48 1617.37 1392.54 1615.49 Q1395.6 1613.62 1399.73 1613.62 Q1405.3 1613.62 1408.33 1617.53 Q1411.35 1621.41 1411.35 1628.61 L1411.35 1650.12 L1405.46 1650.12 L1405.46 1628.8 Q1405.46 1623.67 1403.65 1621.19 Q1401.83 1618.71 1398.11 1618.71 Q1393.56 1618.71 1390.92 1621.73 Q1388.28 1624.76 1388.28 1629.98 L1388.28 1650.12 L1382.39 1650.12 L1382.39 1628.8 Q1382.39 1623.64 1380.57 1621.19 Q1378.76 1618.71 1374.97 1618.71 Q1370.48 1618.71 1367.84 1621.76 Q1365.2 1624.79 1365.2 1629.98 L1365.2 1650.12 L1359.31 1650.12 L1359.31 1614.48 L1365.2 1614.48 L1365.2 1620.01 Q1367.2 1616.74 1370.01 1615.18 Q1372.81 1613.62 1376.66 1613.62 Q1380.54 1613.62 1383.25 1615.59 Q1385.98 1617.56 1387.29 1621.32 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="102.74,1313.94 2352.76,1313.94 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="102.74,1081.37 2352.76,1081.37 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="102.74,848.804 2352.76,848.804 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="102.74,616.235 2352.76,616.235 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="102.74,383.665 2352.76,383.665 "/>
<polyline clip-path="url(#clip312)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="102.74,151.096 2352.76,151.096 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="102.74,1505.26 102.74,47.2441 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="102.74,1313.94 121.638,1313.94 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="102.74,1081.37 121.638,1081.37 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="102.74,848.804 121.638,848.804 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="102.74,616.235 121.638,616.235 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="102.74,383.665 121.638,383.665 "/>
<polyline clip-path="url(#clip310)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="102.74,151.096 121.638,151.096 "/>
<path clip-path="url(#clip310)" d="M-70.3656 1314.39 L-40.6898 1314.39 L-40.6898 1318.33 L-70.3656 1318.33 L-70.3656 1314.39 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-29.7871 1327.29 L-22.1483 1327.29 L-22.1483 1300.92 L-30.4584 1302.59 L-30.4584 1298.33 L-22.1946 1296.66 L-17.5187 1296.66 L-17.5187 1327.29 L-9.8798 1327.29 L-9.8798 1331.22 L-29.7871 1331.22 L-29.7871 1327.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-0.389112 1296.66 L17.9673 1296.66 L17.9673 1300.6 L3.89327 1300.6 L3.89327 1309.07 Q4.91179 1308.72 5.9303 1308.56 Q6.94881 1308.38 7.96732 1308.38 Q13.7543 1308.38 17.1339 1311.55 Q20.5136 1314.72 20.5136 1320.14 Q20.5136 1325.71 17.0413 1328.82 Q13.5691 1331.89 7.24974 1331.89 Q5.07382 1331.89 2.80531 1331.52 Q0.559957 1331.15 -1.84744 1330.41 L-1.84744 1325.71 Q0.235884 1326.85 2.45809 1327.4 Q4.6803 1327.96 7.15714 1327.96 Q11.1618 1327.96 13.4997 1325.85 Q15.8377 1323.75 15.8377 1320.14 Q15.8377 1316.52 13.4997 1314.42 Q11.1618 1312.31 7.15714 1312.31 Q5.28215 1312.31 3.40716 1312.73 Q1.55532 1313.14 -0.389112 1314.02 L-0.389112 1296.66 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M29.7265 1325.34 L34.6107 1325.34 L34.6107 1331.22 L29.7265 1331.22 L29.7265 1325.34 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M54.7958 1299.74 Q51.1847 1299.74 49.356 1303.31 Q47.5504 1306.85 47.5504 1313.98 Q47.5504 1321.08 49.356 1324.65 Q51.1847 1328.19 54.7958 1328.19 Q58.43 1328.19 60.2356 1324.65 Q62.0643 1321.08 62.0643 1313.98 Q62.0643 1306.85 60.2356 1303.31 Q58.43 1299.74 54.7958 1299.74 M54.7958 1296.04 Q60.6059 1296.04 63.6615 1300.65 Q66.7402 1305.23 66.7402 1313.98 Q66.7402 1322.71 63.6615 1327.31 Q60.6059 1331.89 54.7958 1331.89 Q48.9856 1331.89 45.9069 1327.31 Q42.8514 1322.71 42.8514 1313.98 Q42.8514 1305.23 45.9069 1300.65 Q48.9856 1296.04 54.7958 1296.04 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-69.3702 1081.83 L-39.6945 1081.83 L-39.6945 1085.76 L-69.3702 1085.76 L-69.3702 1081.83 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-28.7917 1094.72 L-21.1529 1094.72 L-21.1529 1068.35 L-29.463 1070.02 L-29.463 1065.76 L-21.1992 1064.09 L-16.5233 1064.09 L-16.5233 1094.72 L-8.88444 1094.72 L-8.88444 1098.65 L-28.7917 1098.65 L-28.7917 1094.72 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M13.4071 1068.17 L1.60162 1086.62 L13.4071 1086.62 L13.4071 1068.17 M12.1803 1064.09 L18.0599 1064.09 L18.0599 1086.62 L22.9904 1086.62 L22.9904 1090.51 L18.0599 1090.51 L18.0599 1098.65 L13.4071 1098.65 L13.4071 1090.51 L-2.19466 1090.51 L-2.19466 1085.99 L12.1803 1064.09 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M30.7218 1092.77 L35.6061 1092.77 L35.6061 1098.65 L30.7218 1098.65 L30.7218 1092.77 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M45.8375 1064.09 L64.1939 1064.09 L64.1939 1068.03 L50.1199 1068.03 L50.1199 1076.5 Q51.1384 1076.15 52.1569 1075.99 Q53.1754 1075.81 54.1939 1075.81 Q59.9809 1075.81 63.3605 1078.98 Q66.7402 1082.15 66.7402 1087.57 Q66.7402 1093.14 63.268 1096.25 Q59.7958 1099.33 53.4763 1099.33 Q51.3004 1099.33 49.0319 1098.95 Q46.7866 1098.58 44.3792 1097.84 L44.3792 1093.14 Q46.4625 1094.28 48.6847 1094.83 Q50.9069 1095.39 53.3837 1095.39 Q57.3884 1095.39 59.7263 1093.28 Q62.0643 1091.18 62.0643 1087.57 Q62.0643 1083.96 59.7263 1081.85 Q57.3884 1079.74 53.3837 1079.74 Q51.5088 1079.74 49.6338 1080.16 Q47.7819 1080.58 45.8375 1081.46 L45.8375 1064.09 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-70.3656 849.256 L-40.6898 849.256 L-40.6898 853.191 L-70.3656 853.191 L-70.3656 849.256 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-29.7871 862.149 L-22.1483 862.149 L-22.1483 835.784 L-30.4584 837.45 L-30.4584 833.191 L-22.1946 831.524 L-17.5187 831.524 L-17.5187 862.149 L-9.8798 862.149 L-9.8798 866.084 L-29.7871 866.084 L-29.7871 862.149 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M12.4117 835.598 L0.606253 854.047 L12.4117 854.047 L12.4117 835.598 M11.1849 831.524 L17.0645 831.524 L17.0645 854.047 L21.995 854.047 L21.995 857.936 L17.0645 857.936 L17.0645 866.084 L12.4117 866.084 L12.4117 857.936 L-3.19002 857.936 L-3.19002 853.422 L11.1849 831.524 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M29.7265 860.205 L34.6107 860.205 L34.6107 866.084 L29.7265 866.084 L29.7265 860.205 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M54.7958 834.603 Q51.1847 834.603 49.356 838.168 Q47.5504 841.71 47.5504 848.839 Q47.5504 855.946 49.356 859.51 Q51.1847 863.052 54.7958 863.052 Q58.43 863.052 60.2356 859.51 Q62.0643 855.946 62.0643 848.839 Q62.0643 841.71 60.2356 838.168 Q58.43 834.603 54.7958 834.603 M54.7958 830.899 Q60.6059 830.899 63.6615 835.506 Q66.7402 840.089 66.7402 848.839 Q66.7402 857.566 63.6615 862.172 Q60.6059 866.756 54.7958 866.756 Q48.9856 866.756 45.9069 862.172 Q42.8514 857.566 42.8514 848.839 Q42.8514 840.089 45.9069 835.506 Q48.9856 830.899 54.7958 830.899 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-69.3702 616.686 L-39.6945 616.686 L-39.6945 620.621 L-69.3702 620.621 L-69.3702 616.686 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-28.7917 629.58 L-21.1529 629.58 L-21.1529 603.214 L-29.463 604.881 L-29.463 600.621 L-21.1992 598.955 L-16.5233 598.955 L-16.5233 629.58 L-8.88444 629.58 L-8.88444 633.515 L-28.7917 633.515 L-28.7917 629.58 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M14.7265 614.881 Q18.083 615.598 19.958 617.867 Q21.8561 620.135 21.8561 623.469 Q21.8561 628.584 18.3376 631.385 Q14.8191 634.186 8.33769 634.186 Q6.16178 634.186 3.84698 633.746 Q1.55532 633.33 -0.898369 632.473 L-0.898369 627.959 Q1.04607 629.094 3.36087 629.672 Q5.67567 630.251 8.1988 630.251 Q12.5969 630.251 14.8886 628.515 Q17.2034 626.779 17.2034 623.469 Q17.2034 620.413 15.0506 618.7 Q12.921 616.964 9.10158 616.964 L5.07382 616.964 L5.07382 613.121 L9.28676 613.121 Q12.7358 613.121 14.5645 611.756 Q16.3932 610.367 16.3932 607.774 Q16.3932 605.112 14.4951 603.7 Q12.6201 602.265 9.10158 602.265 Q7.18029 602.265 4.98123 602.682 Q2.78217 603.098 0.143292 603.978 L0.143292 599.811 Q2.80531 599.071 5.12012 598.7 Q7.45807 598.33 9.51824 598.33 Q14.8423 598.33 17.9441 600.76 Q21.046 603.168 21.046 607.288 Q21.046 610.158 19.4024 612.149 Q17.7589 614.117 14.7265 614.881 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M30.7218 627.635 L35.6061 627.635 L35.6061 633.515 L30.7218 633.515 L30.7218 627.635 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M45.8375 598.955 L64.1939 598.955 L64.1939 602.89 L50.1199 602.89 L50.1199 611.362 Q51.1384 611.015 52.1569 610.853 Q53.1754 610.668 54.1939 610.668 Q59.9809 610.668 63.3605 613.839 Q66.7402 617.01 66.7402 622.427 Q66.7402 628.006 63.268 631.107 Q59.7958 634.186 53.4763 634.186 Q51.3004 634.186 49.0319 633.816 Q46.7866 633.445 44.3792 632.705 L44.3792 628.006 Q46.4625 629.14 48.6847 629.695 Q50.9069 630.251 53.3837 630.251 Q57.3884 630.251 59.7263 628.144 Q62.0643 626.038 62.0643 622.427 Q62.0643 618.816 59.7263 616.709 Q57.3884 614.603 53.3837 614.603 Q51.5088 614.603 49.6338 615.02 Q47.7819 615.436 45.8375 616.316 L45.8375 598.955 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-70.3656 384.117 L-40.6898 384.117 L-40.6898 388.052 L-70.3656 388.052 L-70.3656 384.117 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-29.7871 397.01 L-22.1483 397.01 L-22.1483 370.644 L-30.4584 372.311 L-30.4584 368.052 L-22.1946 366.385 L-17.5187 366.385 L-17.5187 397.01 L-9.8798 397.01 L-9.8798 400.945 L-29.7871 400.945 L-29.7871 397.01 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M13.7312 382.311 Q17.0876 383.029 18.9626 385.297 Q20.8608 387.566 20.8608 390.899 Q20.8608 396.015 17.3423 398.816 Q13.8238 401.617 7.34233 401.617 Q5.16641 401.617 2.85161 401.177 Q0.559957 400.76 -1.89373 399.904 L-1.89373 395.39 Q0.0507 396.524 2.3655 397.103 Q4.6803 397.681 7.20344 397.681 Q11.6016 397.681 13.8932 395.945 Q16.208 394.209 16.208 390.899 Q16.208 387.843 14.0553 386.131 Q11.9256 384.394 8.10621 384.394 L4.07846 384.394 L4.07846 380.552 L8.2914 380.552 Q11.7405 380.552 13.5691 379.186 Q15.3978 377.797 15.3978 375.205 Q15.3978 372.543 13.4997 371.131 Q11.6247 369.695 8.10621 369.695 Q6.18493 369.695 3.98586 370.112 Q1.7868 370.529 -0.852073 371.408 L-0.852073 367.242 Q1.80995 366.501 4.12475 366.131 Q6.4627 365.76 8.52288 365.76 Q13.8469 365.76 16.9488 368.191 Q20.0506 370.598 20.0506 374.719 Q20.0506 377.589 18.4071 379.58 Q16.7636 381.547 13.7312 382.311 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M29.7265 395.066 L34.6107 395.066 L34.6107 400.945 L29.7265 400.945 L29.7265 395.066 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M54.7958 369.464 Q51.1847 369.464 49.356 373.029 Q47.5504 376.57 47.5504 383.7 Q47.5504 390.806 49.356 394.371 Q51.1847 397.913 54.7958 397.913 Q58.43 397.913 60.2356 394.371 Q62.0643 390.806 62.0643 383.7 Q62.0643 376.57 60.2356 373.029 Q58.43 369.464 54.7958 369.464 M54.7958 365.76 Q60.6059 365.76 63.6615 370.367 Q66.7402 374.95 66.7402 383.7 Q66.7402 392.427 63.6615 397.033 Q60.6059 401.617 54.7958 401.617 Q48.9856 401.617 45.9069 397.033 Q42.8514 392.427 42.8514 383.7 Q42.8514 374.95 45.9069 370.367 Q48.9856 365.76 54.7958 365.76 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-69.3702 151.547 L-39.6945 151.547 L-39.6945 155.482 L-69.3702 155.482 L-69.3702 151.547 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-28.7917 164.44 L-21.1529 164.44 L-21.1529 138.075 L-29.463 139.742 L-29.463 135.482 L-21.1992 133.816 L-16.5233 133.816 L-16.5233 164.44 L-8.88444 164.44 L-8.88444 168.376 L-28.7917 168.376 L-28.7917 164.44 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M4.58771 164.44 L20.9071 164.44 L20.9071 168.376 L-1.03726 168.376 L-1.03726 164.44 Q1.62477 161.686 6.20807 157.056 Q10.8145 152.404 11.9951 151.061 Q14.2404 148.538 15.1201 146.802 Q16.0228 145.042 16.0228 143.353 Q16.0228 140.598 14.0784 138.862 Q12.1571 137.126 9.05528 137.126 Q6.85622 137.126 4.40253 137.89 Q1.97199 138.654 -0.805777 140.205 L-0.805777 135.482 Q2.01828 134.348 4.47197 133.769 Q6.92566 133.191 8.96269 133.191 Q14.333 133.191 17.5275 135.876 Q20.7219 138.561 20.7219 143.052 Q20.7219 145.181 19.9117 147.103 Q19.1247 149.001 17.0182 151.593 Q16.4395 152.265 13.3377 155.482 Q10.2358 158.677 4.58771 164.44 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M30.7218 162.496 L35.6061 162.496 L35.6061 168.376 L30.7218 168.376 L30.7218 162.496 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M45.8375 133.816 L64.1939 133.816 L64.1939 137.751 L50.1199 137.751 L50.1199 146.223 Q51.1384 145.876 52.1569 145.714 Q53.1754 145.529 54.1939 145.529 Q59.9809 145.529 63.3605 148.7 Q66.7402 151.871 66.7402 157.288 Q66.7402 162.866 63.268 165.968 Q59.7958 169.047 53.4763 169.047 Q51.3004 169.047 49.0319 168.677 Q46.7866 168.306 44.3792 167.565 L44.3792 162.866 Q46.4625 164.001 48.6847 164.556 Q50.9069 165.112 53.3837 165.112 Q57.3884 165.112 59.7263 163.005 Q62.0643 160.899 62.0643 157.288 Q62.0643 153.677 59.7263 151.57 Q57.3884 149.464 53.3837 149.464 Q51.5088 149.464 49.6338 149.88 Q47.7819 150.297 45.8375 151.177 L45.8375 133.816 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-139.91 854.009 L-137.045 854.009 L-137.045 880.936 Q-130.998 880.554 -127.815 877.308 Q-124.664 874.029 -124.664 868.205 Q-124.664 864.831 -125.491 861.68 Q-126.319 858.497 -127.974 855.378 L-122.436 855.378 Q-121.099 858.529 -120.399 861.839 Q-119.699 865.149 -119.699 868.555 Q-119.699 877.085 -124.664 882.082 Q-129.629 887.047 -138.095 887.047 Q-146.848 887.047 -151.973 882.336 Q-157.129 877.594 -157.129 869.573 Q-157.129 862.38 -152.482 858.21 Q-147.867 854.009 -139.91 854.009 M-141.628 859.866 Q-146.435 859.929 -149.299 862.571 Q-152.164 865.181 -152.164 869.51 Q-152.164 874.411 -149.395 877.371 Q-146.626 880.299 -141.597 880.745 L-141.628 859.866 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-142.138 814.764 L-120.622 814.764 L-120.622 820.621 L-141.947 820.621 Q-147.007 820.621 -149.522 822.594 Q-152.036 824.568 -152.036 828.514 Q-152.036 833.257 -149.013 835.994 Q-145.989 838.731 -140.769 838.731 L-120.622 838.731 L-120.622 844.62 L-156.27 844.62 L-156.27 838.731 L-150.731 838.731 Q-153.946 836.631 -155.537 833.798 Q-157.129 830.933 -157.129 827.209 Q-157.129 821.067 -153.309 817.916 Q-149.522 814.764 -142.138 814.764 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-139.91 772.592 L-137.045 772.592 L-137.045 799.519 Q-130.998 799.137 -127.815 795.89 Q-124.664 792.612 -124.664 786.787 Q-124.664 783.413 -125.491 780.262 Q-126.319 777.08 -127.974 773.96 L-122.436 773.96 Q-121.099 777.111 -120.399 780.422 Q-119.699 783.732 -119.699 787.137 Q-119.699 795.667 -124.664 800.664 Q-129.629 805.63 -138.095 805.63 Q-146.848 805.63 -151.973 800.919 Q-157.129 796.177 -157.129 788.156 Q-157.129 780.963 -152.482 776.793 Q-147.867 772.592 -139.91 772.592 M-141.628 778.448 Q-146.435 778.512 -149.299 781.154 Q-152.164 783.764 -152.164 788.092 Q-152.164 792.994 -149.395 795.954 Q-146.626 798.882 -141.597 799.328 L-141.628 778.448 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-150.795 742.323 Q-151.368 743.309 -151.623 744.487 Q-151.909 745.633 -151.909 747.033 Q-151.909 751.999 -148.663 754.672 Q-145.448 757.314 -139.4 757.314 L-120.622 757.314 L-120.622 763.202 L-156.27 763.202 L-156.27 757.314 L-150.731 757.314 Q-153.978 755.468 -155.537 752.508 Q-157.129 749.548 -157.129 745.315 Q-157.129 744.71 -157.033 743.978 Q-156.97 743.246 -156.811 742.355 L-150.795 742.323 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-138.859 713.868 Q-145.225 713.868 -148.726 716.51 Q-152.227 719.12 -152.227 723.862 Q-152.227 728.573 -148.726 731.215 Q-145.225 733.825 -138.859 733.825 Q-132.525 733.825 -129.024 731.215 Q-125.523 728.573 -125.523 723.862 Q-125.523 719.12 -129.024 716.51 Q-132.525 713.868 -138.859 713.868 M-125.046 708.012 Q-115.943 708.012 -111.519 712.054 Q-107.063 716.096 -107.063 724.435 Q-107.063 727.522 -107.54 730.26 Q-107.986 732.997 -108.941 735.575 L-114.638 735.575 Q-113.237 732.997 -112.569 730.483 Q-111.901 727.968 -111.901 725.358 Q-111.901 719.597 -114.924 716.733 Q-117.916 713.868 -123.995 713.868 L-126.892 713.868 Q-123.741 715.682 -122.181 718.515 Q-120.622 721.348 -120.622 725.294 Q-120.622 731.851 -125.619 735.862 Q-130.616 739.872 -138.859 739.872 Q-147.135 739.872 -152.132 735.862 Q-157.129 731.851 -157.129 725.294 Q-157.129 721.348 -155.569 718.515 Q-154.01 715.682 -150.859 713.868 L-156.27 713.868 L-156.27 708.012 L-125.046 708.012 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip310)" d="M-117.311 681.116 Q-110.946 683.599 -109.004 685.954 Q-107.063 688.31 -107.063 692.256 L-107.063 696.935 L-111.964 696.935 L-111.964 693.498 Q-111.964 691.079 -113.11 689.742 Q-114.256 688.405 -118.521 686.782 L-121.195 685.732 L-156.27 700.15 L-156.27 693.943 L-128.388 682.803 L-156.27 671.663 L-156.27 665.457 L-117.311 681.116 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><circle clip-path="url(#clip312)" cx="1123.22" cy="1464" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="2289.08" cy="1403.02" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="1123.22" cy="978.292" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="2289.08" cy="921.474" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="1123.22" cy="921.474" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="2289.08" cy="509.14" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="2289.08" cy="509.14" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="920.402" cy="509.14" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="1326.04" cy="509.14" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="2289.08" cy="472.743" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="1123.22" cy="472.743" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="2289.08" cy="439.927" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="166.42" cy="88.5087" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="2080.02" cy="88.5087" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="1179.7" cy="88.5087" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip312)" cx="1066.75" cy="88.5087" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
</svg>

```

---

*This page was generated using [Literate.jl](https://github.com/fredrikekre/Literate.jl).*

