```@meta
EditURL = "https://github.com/maartenvd/MPSKit.jl/examples/classic2d/1.hard-hexagon/main.jl"
```

[![](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/maartenvd/MPSKit.jl/gh-pages?filepath=dev/examples/classic2d/1.hard-hexagon/main.ipynb)
[![](https://img.shields.io/badge/show-nbviewer-579ACA.svg)](https://nbviewer.jupyter.org/github/maartenvd/MPSKit.jl/blob/gh-pages/dev/examples/classic2d/1.hard-hexagon/main.ipynb)
[![](https://img.shields.io/badge/download-project-orange)](https://minhaskamal.github.io/DownGit/#/home?url=https://github.com/maartenvd/MPSKit.jl/examples/tree/gh-pages/dev/examples/classic2d/1.hard-hexagon)

# [The Hard Hexagon model](@id demo_hardhexagon)

![logo](hexagon.svg)

Tensor networks are a natural way to do statistical mechanics on a lattice.
As an example of this we will extract the central charge of the hard hexagon model.
This model is known to have central charge 0.8, and has very peculiar non-local (anyonic) symmetries.
Because TensorKit supports anyonic symmetries, so does MPSKit.
To follow the tutorial you need the following packages.

````julia
using MPSKit, MPSKitModels, TensorKit, Plots, Polynomials
import TensorOperations; TensorOperations.disable_cache(); # hide
````

The [hard hexagon model](https://en.wikipedia.org/wiki/Hard_hexagon_model) is a 2-dimensional lattice model of a gas, where particles are allowed to be on the vertices of a triangular lattice, but no two particles may be adjacent.
This can be encoded in a transfer matrix with a local MPO tensor using anyonic symmetries, and the resulting MPO has been implemented in MPSKitModels.

In order to use these anyonic symmetries, we need to generalise the notion of the bond dimension and define how it interacts with the symmetry. Thus, we implement away of converting integers to symmetric spaces of the given dimension, which provides a crude guess for how the final MPS would distribute its Schmidt spectrum.

````julia
mpo = hard_hexagon()
P = space(mpo.opp[1], 2)
function virtual_space(D::Integer)
    _D = round(Int, D / sum(dim, values(FibonacciAnyon)))
    return Vect[FibonacciAnyon](sector => _D for sector in (:I, :τ))
end

@assert isapprox(dim(virtual_space(100)), 100; atol=3)
````

## The leading boundary

One way to study statistical mechanics in infinite systems with tensor networks is by approximating the dominant eigenvector of the transfer matrix by an MPS.
This dominant eigenvector contains a lot of hidden information.
For example, the free energy can be extracted by computing the expectation value of the mpo.
Additionally, we can compute the entanglement entropy as well as the correlation length of the state:

````julia
D = 10
V = virtual_space(D)
ψ₀ = InfiniteMPS([P], [V])
ψ, envs, = leading_boundary(ψ₀, mpo, VUMPS(; verbose=false))
F = real(first(expectation_value(ψ, mpo)))
S = real(first(entropy(ψ)))
ξ = correlation_length(ψ)
println("F = $F\tS = $S\tξ = $ξ")
````

````
F = 0.883903705170386	S = 1.2807829621983375	ξ = 13.849682582471392

````

## The scaling hypothesis

The dominant eigenvector is of course only an approximation. The finite bond dimension enforces a finite correlation length, which effectively introduces a length scale in the system. This can be exploited to formulate a [scaling hypothesis](https://arxiv.org/pdf/0812.2903.pdf), which in turn allows to extract the central charge.

First we need to know the entropy and correlation length at a bunch of different bond dimensions. Our approach will be to re-use the previous approximated dominant eigenvector, and then expanding its bond dimension and re-running VUMPS.
According to the scaling hypothesis we should have ``S \propto \frac{c}{6} log(ξ)``. Therefore we should find ``c`` using

````julia
function scaling_simulations(ψ₀, mpo, Ds; verbose=false, tol=1e-6)
    entropies = similar(Ds, Float64)
    correlations = similar(Ds, Float64)
    alg = VUMPS(; verbose=verbose, tol_galerkin=tol)

    ψ, envs, = leading_boundary(ψ₀, mpo, alg)
    entropies[1] = real(entropy(ψ)[1])
    correlations[1] = correlation_length(ψ)

    for (i, d) in enumerate(diff(Ds))
        ψ, envs = changebonds(ψ, mpo, OptimalExpand(; trscheme=truncdim(d)), envs)
        ψ, envs, = leading_boundary(ψ, mpo, alg, envs)
        entropies[i+1] = real(entropy(ψ)[1])
        correlations[i+1] = correlation_length(ψ)
    end
    return entropies, correlations
end


bond_dimensions = 10:5:25
ψ₀ = InfiniteMPS([P], [virtual_space(bond_dimensions[1])])
Ss, ξs = scaling_simulations(ψ₀, mpo, bond_dimensions)

f = fit(log.(ξs), 6 * Ss, 1)
c = f.coeffs[2]
````

````
0.8025228820464224
````

````julia
p = plot(; xlabel="logarithmic correlation length", ylabel="entanglement entropy")
p = plot(log.(ξs), Ss; seriestype=:scatter, label=nothing)
plot!(p, ξ -> f(ξ) / 6; label="fit")
````

```@raw html
<?xml version="1.0" encoding="utf-8"?>
<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="600" height="400" viewBox="0 0 2400 1600">
<defs>
  <clipPath id="clip220">
    <rect x="0" y="0" width="2400" height="1600"/>
  </clipPath>
</defs>
<path clip-path="url(#clip220)" d="M0 1600 L2400 1600 L2400 0 L0 0  Z" fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip221">
    <rect x="480" y="0" width="1681" height="1600"/>
  </clipPath>
</defs>
<path clip-path="url(#clip220)" d="M184.191 1486.45 L2352.76 1486.45 L2352.76 47.2441 L184.191 47.2441  Z" fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<defs>
  <clipPath id="clip222">
    <rect x="184" y="47" width="2170" height="1440"/>
  </clipPath>
</defs>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="401.779,1486.45 401.779,47.2441 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="722.585,1486.45 722.585,47.2441 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1043.39,1486.45 1043.39,47.2441 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1364.2,1486.45 1364.2,47.2441 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="1685,1486.45 1685,47.2441 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="2005.81,1486.45 2005.81,47.2441 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="2326.62,1486.45 2326.62,47.2441 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="184.191,1486.45 2352.76,1486.45 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="401.779,1486.45 401.779,1467.55 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="722.585,1486.45 722.585,1467.55 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1043.39,1486.45 1043.39,1467.55 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1364.2,1486.45 1364.2,1467.55 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="1685,1486.45 1685,1467.55 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="2005.81,1486.45 2005.81,1467.55 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="2326.62,1486.45 2326.62,1467.55 "/>
<path clip-path="url(#clip220)" d="M358.435 1544.91 L374.754 1544.91 L374.754 1548.85 L352.81 1548.85 L352.81 1544.91 Q355.472 1542.16 360.055 1537.53 Q364.661 1532.88 365.842 1531.53 Q368.087 1529.01 368.967 1527.27 Q369.87 1525.51 369.87 1523.82 Q369.87 1521.07 367.925 1519.33 Q366.004 1517.6 362.902 1517.6 Q360.703 1517.6 358.249 1518.36 Q355.819 1519.13 353.041 1520.68 L353.041 1515.95 Q355.865 1514.82 358.319 1514.24 Q360.773 1513.66 362.81 1513.66 Q368.18 1513.66 371.374 1516.35 Q374.569 1519.03 374.569 1523.52 Q374.569 1525.65 373.759 1527.57 Q372.972 1529.47 370.865 1532.07 Q370.286 1532.74 367.185 1535.95 Q364.083 1539.15 358.435 1544.91 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M384.569 1542.97 L389.453 1542.97 L389.453 1548.85 L384.569 1548.85 L384.569 1542.97 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M398.458 1514.29 L420.68 1514.29 L420.68 1516.28 L408.133 1548.85 L403.249 1548.85 L415.055 1518.22 L398.458 1518.22 L398.458 1514.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M429.846 1514.29 L448.203 1514.29 L448.203 1518.22 L434.129 1518.22 L434.129 1526.7 Q435.147 1526.35 436.166 1526.19 Q437.184 1526 438.203 1526 Q443.99 1526 447.369 1529.17 Q450.749 1532.34 450.749 1537.76 Q450.749 1543.34 447.277 1546.44 Q443.805 1549.52 437.485 1549.52 Q435.309 1549.52 433.041 1549.15 Q430.795 1548.78 428.388 1548.04 L428.388 1543.34 Q430.471 1544.47 432.693 1545.03 Q434.916 1545.58 437.393 1545.58 Q441.397 1545.58 443.735 1543.48 Q446.073 1541.37 446.073 1537.76 Q446.073 1534.15 443.735 1532.04 Q441.397 1529.94 437.393 1529.94 Q435.518 1529.94 433.643 1530.35 Q431.791 1530.77 429.846 1531.65 L429.846 1514.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M688.812 1530.21 Q692.169 1530.93 694.044 1533.2 Q695.942 1535.47 695.942 1538.8 Q695.942 1543.92 692.423 1546.72 Q688.905 1549.52 682.424 1549.52 Q680.248 1549.52 677.933 1549.08 Q675.641 1548.66 673.187 1547.81 L673.187 1543.29 Q675.132 1544.43 677.447 1545.01 Q679.762 1545.58 682.285 1545.58 Q686.683 1545.58 688.974 1543.85 Q691.289 1542.11 691.289 1538.8 Q691.289 1535.75 689.136 1534.03 Q687.007 1532.3 683.187 1532.3 L679.16 1532.3 L679.16 1528.45 L683.373 1528.45 Q686.822 1528.45 688.65 1527.09 Q690.479 1525.7 690.479 1523.11 Q690.479 1520.45 688.581 1519.03 Q686.706 1517.6 683.187 1517.6 Q681.266 1517.6 679.067 1518.01 Q676.868 1518.43 674.229 1519.31 L674.229 1515.14 Q676.891 1514.4 679.206 1514.03 Q681.544 1513.66 683.604 1513.66 Q688.928 1513.66 692.03 1516.09 Q695.132 1518.5 695.132 1522.62 Q695.132 1525.49 693.488 1527.48 Q691.845 1529.45 688.812 1530.21 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M704.808 1542.97 L709.692 1542.97 L709.692 1548.85 L704.808 1548.85 L704.808 1542.97 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M729.877 1517.37 Q726.266 1517.37 724.437 1520.93 Q722.632 1524.47 722.632 1531.6 Q722.632 1538.71 724.437 1542.27 Q726.266 1545.82 729.877 1545.82 Q733.511 1545.82 735.317 1542.27 Q737.145 1538.71 737.145 1531.6 Q737.145 1524.47 735.317 1520.93 Q733.511 1517.37 729.877 1517.37 M729.877 1513.66 Q735.687 1513.66 738.743 1518.27 Q741.821 1522.85 741.821 1531.6 Q741.821 1540.33 738.743 1544.94 Q735.687 1549.52 729.877 1549.52 Q724.067 1549.52 720.988 1544.94 Q717.933 1540.33 717.933 1531.6 Q717.933 1522.85 720.988 1518.27 Q724.067 1513.66 729.877 1513.66 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M760.039 1517.37 Q756.428 1517.37 754.599 1520.93 Q752.794 1524.47 752.794 1531.6 Q752.794 1538.71 754.599 1542.27 Q756.428 1545.82 760.039 1545.82 Q763.673 1545.82 765.479 1542.27 Q767.307 1538.71 767.307 1531.6 Q767.307 1524.47 765.479 1520.93 Q763.673 1517.37 760.039 1517.37 M760.039 1513.66 Q765.849 1513.66 768.905 1518.27 Q771.983 1522.85 771.983 1531.6 Q771.983 1540.33 768.905 1544.94 Q765.849 1549.52 760.039 1549.52 Q754.229 1549.52 751.15 1544.94 Q748.094 1540.33 748.094 1531.6 Q748.094 1522.85 751.15 1518.27 Q754.229 1513.66 760.039 1513.66 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1010.12 1530.21 Q1013.47 1530.93 1015.35 1533.2 Q1017.25 1535.47 1017.25 1538.8 Q1017.25 1543.92 1013.73 1546.72 Q1010.21 1549.52 1003.73 1549.52 Q1001.55 1549.52 999.237 1549.08 Q996.945 1548.66 994.491 1547.81 L994.491 1543.29 Q996.436 1544.43 998.75 1545.01 Q1001.07 1545.58 1003.59 1545.58 Q1007.99 1545.58 1010.28 1543.85 Q1012.59 1542.11 1012.59 1538.8 Q1012.59 1535.75 1010.44 1534.03 Q1008.31 1532.3 1004.49 1532.3 L1000.46 1532.3 L1000.46 1528.45 L1004.68 1528.45 Q1008.13 1528.45 1009.95 1527.09 Q1011.78 1525.7 1011.78 1523.11 Q1011.78 1520.45 1009.88 1519.03 Q1008.01 1517.6 1004.49 1517.6 Q1002.57 1517.6 1000.37 1518.01 Q998.172 1518.43 995.533 1519.31 L995.533 1515.14 Q998.195 1514.4 1000.51 1514.03 Q1002.85 1513.66 1004.91 1513.66 Q1010.23 1513.66 1013.33 1516.09 Q1016.44 1518.5 1016.44 1522.62 Q1016.44 1525.49 1014.79 1527.48 Q1013.15 1529.45 1010.12 1530.21 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1026.11 1542.97 L1031 1542.97 L1031 1548.85 L1026.11 1548.85 L1026.11 1542.97 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1045.21 1544.91 L1061.53 1544.91 L1061.53 1548.85 L1039.58 1548.85 L1039.58 1544.91 Q1042.25 1542.16 1046.83 1537.53 Q1051.44 1532.88 1052.62 1531.53 Q1054.86 1529.01 1055.74 1527.27 Q1056.64 1525.51 1056.64 1523.82 Q1056.64 1521.07 1054.7 1519.33 Q1052.78 1517.6 1049.68 1517.6 Q1047.48 1517.6 1045.02 1518.36 Q1042.59 1519.13 1039.82 1520.68 L1039.82 1515.95 Q1042.64 1514.82 1045.09 1514.24 Q1047.55 1513.66 1049.58 1513.66 Q1054.95 1513.66 1058.15 1516.35 Q1061.34 1519.03 1061.34 1523.52 Q1061.34 1525.65 1060.53 1527.57 Q1059.75 1529.47 1057.64 1532.07 Q1057.06 1532.74 1053.96 1535.95 Q1050.86 1539.15 1045.21 1544.91 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1071.39 1514.29 L1089.75 1514.29 L1089.75 1518.22 L1075.67 1518.22 L1075.67 1526.7 Q1076.69 1526.35 1077.71 1526.19 Q1078.73 1526 1079.75 1526 Q1085.53 1526 1088.91 1529.17 Q1092.29 1532.34 1092.29 1537.76 Q1092.29 1543.34 1088.82 1546.44 Q1085.35 1549.52 1079.03 1549.52 Q1076.85 1549.52 1074.58 1549.15 Q1072.34 1548.78 1069.93 1548.04 L1069.93 1543.34 Q1072.01 1544.47 1074.24 1545.03 Q1076.46 1545.58 1078.94 1545.58 Q1082.94 1545.58 1085.28 1543.48 Q1087.62 1541.37 1087.62 1537.76 Q1087.62 1534.15 1085.28 1532.04 Q1082.94 1529.94 1078.94 1529.94 Q1077.06 1529.94 1075.19 1530.35 Q1073.33 1530.77 1071.39 1531.65 L1071.39 1514.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1330.42 1530.21 Q1333.78 1530.93 1335.66 1533.2 Q1337.55 1535.47 1337.55 1538.8 Q1337.55 1543.92 1334.04 1546.72 Q1330.52 1549.52 1324.04 1549.52 Q1321.86 1549.52 1319.54 1549.08 Q1317.25 1548.66 1314.8 1547.81 L1314.8 1543.29 Q1316.74 1544.43 1319.06 1545.01 Q1321.37 1545.58 1323.9 1545.58 Q1328.29 1545.58 1330.59 1543.85 Q1332.9 1542.11 1332.9 1538.8 Q1332.9 1535.75 1330.75 1534.03 Q1328.62 1532.3 1324.8 1532.3 L1320.77 1532.3 L1320.77 1528.45 L1324.98 1528.45 Q1328.43 1528.45 1330.26 1527.09 Q1332.09 1525.7 1332.09 1523.11 Q1332.09 1520.45 1330.19 1519.03 Q1328.32 1517.6 1324.8 1517.6 Q1322.88 1517.6 1320.68 1518.01 Q1318.48 1518.43 1315.84 1519.31 L1315.84 1515.14 Q1318.5 1514.4 1320.82 1514.03 Q1323.16 1513.66 1325.22 1513.66 Q1330.54 1513.66 1333.64 1516.09 Q1336.74 1518.5 1336.74 1522.62 Q1336.74 1525.49 1335.1 1527.48 Q1333.46 1529.45 1330.42 1530.21 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1346.42 1542.97 L1351.3 1542.97 L1351.3 1548.85 L1346.42 1548.85 L1346.42 1542.97 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1361.54 1514.29 L1379.89 1514.29 L1379.89 1518.22 L1365.82 1518.22 L1365.82 1526.7 Q1366.84 1526.35 1367.85 1526.19 Q1368.87 1526 1369.89 1526 Q1375.68 1526 1379.06 1529.17 Q1382.44 1532.34 1382.44 1537.76 Q1382.44 1543.34 1378.97 1546.44 Q1375.49 1549.52 1369.17 1549.52 Q1367 1549.52 1364.73 1549.15 Q1362.48 1548.78 1360.08 1548.04 L1360.08 1543.34 Q1362.16 1544.47 1364.38 1545.03 Q1366.6 1545.58 1369.08 1545.58 Q1373.09 1545.58 1375.42 1543.48 Q1377.76 1541.37 1377.76 1537.76 Q1377.76 1534.15 1375.42 1532.04 Q1373.09 1529.94 1369.08 1529.94 Q1367.21 1529.94 1365.33 1530.35 Q1363.48 1530.77 1361.54 1531.65 L1361.54 1514.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1401.65 1517.37 Q1398.04 1517.37 1396.21 1520.93 Q1394.41 1524.47 1394.41 1531.6 Q1394.41 1538.71 1396.21 1542.27 Q1398.04 1545.82 1401.65 1545.82 Q1405.29 1545.82 1407.09 1542.27 Q1408.92 1538.71 1408.92 1531.6 Q1408.92 1524.47 1407.09 1520.93 Q1405.29 1517.37 1401.65 1517.37 M1401.65 1513.66 Q1407.46 1513.66 1410.52 1518.27 Q1413.6 1522.85 1413.6 1531.6 Q1413.6 1540.33 1410.52 1544.94 Q1407.46 1549.52 1401.65 1549.52 Q1395.84 1549.52 1392.76 1544.94 Q1389.71 1540.33 1389.71 1531.6 Q1389.71 1522.85 1392.76 1518.27 Q1395.84 1513.66 1401.65 1513.66 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1651.73 1530.21 Q1655.08 1530.93 1656.96 1533.2 Q1658.86 1535.47 1658.86 1538.8 Q1658.86 1543.92 1655.34 1546.72 Q1651.82 1549.52 1645.34 1549.52 Q1643.16 1549.52 1640.85 1549.08 Q1638.56 1548.66 1636.1 1547.81 L1636.1 1543.29 Q1638.05 1544.43 1640.36 1545.01 Q1642.68 1545.58 1645.2 1545.58 Q1649.6 1545.58 1651.89 1543.85 Q1654.21 1542.11 1654.21 1538.8 Q1654.21 1535.75 1652.05 1534.03 Q1649.92 1532.3 1646.1 1532.3 L1642.08 1532.3 L1642.08 1528.45 L1646.29 1528.45 Q1649.74 1528.45 1651.57 1527.09 Q1653.39 1525.7 1653.39 1523.11 Q1653.39 1520.45 1651.5 1519.03 Q1649.62 1517.6 1646.1 1517.6 Q1644.18 1517.6 1641.98 1518.01 Q1639.78 1518.43 1637.15 1519.31 L1637.15 1515.14 Q1639.81 1514.4 1642.12 1514.03 Q1644.46 1513.66 1646.52 1513.66 Q1651.84 1513.66 1654.95 1516.09 Q1658.05 1518.5 1658.05 1522.62 Q1658.05 1525.49 1656.4 1527.48 Q1654.76 1529.45 1651.73 1530.21 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1667.72 1542.97 L1672.61 1542.97 L1672.61 1548.85 L1667.72 1548.85 L1667.72 1542.97 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1681.61 1514.29 L1703.83 1514.29 L1703.83 1516.28 L1691.29 1548.85 L1686.4 1548.85 L1698.21 1518.22 L1681.61 1518.22 L1681.61 1514.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1713 1514.29 L1731.36 1514.29 L1731.36 1518.22 L1717.28 1518.22 L1717.28 1526.7 Q1718.3 1526.35 1719.32 1526.19 Q1720.34 1526 1721.36 1526 Q1727.14 1526 1730.52 1529.17 Q1733.9 1532.34 1733.9 1537.76 Q1733.9 1543.34 1730.43 1546.44 Q1726.96 1549.52 1720.64 1549.52 Q1718.46 1549.52 1716.2 1549.15 Q1713.95 1548.78 1711.54 1548.04 L1711.54 1543.34 Q1713.63 1544.47 1715.85 1545.03 Q1718.07 1545.58 1720.55 1545.58 Q1724.55 1545.58 1726.89 1543.48 Q1729.23 1541.37 1729.23 1537.76 Q1729.23 1534.15 1726.89 1532.04 Q1724.55 1529.94 1720.55 1529.94 Q1718.67 1529.94 1716.8 1530.35 Q1714.95 1530.77 1713 1531.65 L1713 1514.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1971.37 1518.36 L1959.56 1536.81 L1971.37 1536.81 L1971.37 1518.36 M1970.14 1514.29 L1976.02 1514.29 L1976.02 1536.81 L1980.95 1536.81 L1980.95 1540.7 L1976.02 1540.7 L1976.02 1548.85 L1971.37 1548.85 L1971.37 1540.7 L1955.76 1540.7 L1955.76 1536.19 L1970.14 1514.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M1988.68 1542.97 L1993.56 1542.97 L1993.56 1548.85 L1988.68 1548.85 L1988.68 1542.97 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M2013.75 1517.37 Q2010.14 1517.37 2008.31 1520.93 Q2006.5 1524.47 2006.5 1531.6 Q2006.5 1538.71 2008.31 1542.27 Q2010.14 1545.82 2013.75 1545.82 Q2017.38 1545.82 2019.19 1542.27 Q2021.02 1538.71 2021.02 1531.6 Q2021.02 1524.47 2019.19 1520.93 Q2017.38 1517.37 2013.75 1517.37 M2013.75 1513.66 Q2019.56 1513.66 2022.62 1518.27 Q2025.69 1522.85 2025.69 1531.6 Q2025.69 1540.33 2022.62 1544.94 Q2019.56 1549.52 2013.75 1549.52 Q2007.94 1549.52 2004.86 1544.94 Q2001.8 1540.33 2001.8 1531.6 Q2001.8 1522.85 2004.86 1518.27 Q2007.94 1513.66 2013.75 1513.66 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M2043.91 1517.37 Q2040.3 1517.37 2038.47 1520.93 Q2036.67 1524.47 2036.67 1531.6 Q2036.67 1538.71 2038.47 1542.27 Q2040.3 1545.82 2043.91 1545.82 Q2047.55 1545.82 2049.35 1542.27 Q2051.18 1538.71 2051.18 1531.6 Q2051.18 1524.47 2049.35 1520.93 Q2047.55 1517.37 2043.91 1517.37 M2043.91 1513.66 Q2049.72 1513.66 2052.78 1518.27 Q2055.86 1522.85 2055.86 1531.6 Q2055.86 1540.33 2052.78 1544.94 Q2049.72 1549.52 2043.91 1549.52 Q2038.1 1549.52 2035.02 1544.94 Q2031.97 1540.33 2031.97 1531.6 Q2031.97 1522.85 2035.02 1518.27 Q2038.1 1513.66 2043.91 1513.66 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M2292.67 1518.36 L2280.86 1536.81 L2292.67 1536.81 L2292.67 1518.36 M2291.44 1514.29 L2297.32 1514.29 L2297.32 1536.81 L2302.25 1536.81 L2302.25 1540.7 L2297.32 1540.7 L2297.32 1548.85 L2292.67 1548.85 L2292.67 1540.7 L2277.07 1540.7 L2277.07 1536.19 L2291.44 1514.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M2309.98 1542.97 L2314.87 1542.97 L2314.87 1548.85 L2309.98 1548.85 L2309.98 1542.97 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M2329.08 1544.91 L2345.4 1544.91 L2345.4 1548.85 L2323.46 1548.85 L2323.46 1544.91 Q2326.12 1542.16 2330.7 1537.53 Q2335.31 1532.88 2336.49 1531.53 Q2338.73 1529.01 2339.61 1527.27 Q2340.52 1525.51 2340.52 1523.82 Q2340.52 1521.07 2338.57 1519.33 Q2336.65 1517.6 2333.55 1517.6 Q2331.35 1517.6 2328.9 1518.36 Q2326.47 1519.13 2323.69 1520.68 L2323.69 1515.95 Q2326.51 1514.82 2328.97 1514.24 Q2331.42 1513.66 2333.46 1513.66 Q2338.83 1513.66 2342.02 1516.35 Q2345.22 1519.03 2345.22 1523.52 Q2345.22 1525.65 2344.4 1527.57 Q2343.62 1529.47 2341.51 1532.07 Q2340.93 1532.74 2337.83 1535.95 Q2334.73 1539.15 2329.08 1544.91 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M2355.26 1514.29 L2373.62 1514.29 L2373.62 1518.22 L2359.54 1518.22 L2359.54 1526.7 Q2360.56 1526.35 2361.58 1526.19 Q2362.6 1526 2363.62 1526 Q2369.4 1526 2372.78 1529.17 Q2376.16 1532.34 2376.16 1537.76 Q2376.16 1543.34 2372.69 1546.44 Q2369.22 1549.52 2362.9 1549.52 Q2360.72 1549.52 2358.46 1549.15 Q2356.21 1548.78 2353.8 1548.04 L2353.8 1543.34 Q2355.89 1544.47 2358.11 1545.03 Q2360.33 1545.58 2362.81 1545.58 Q2366.81 1545.58 2369.15 1543.48 Q2371.49 1541.37 2371.49 1537.76 Q2371.49 1534.15 2369.15 1532.04 Q2366.81 1529.94 2362.81 1529.94 Q2360.93 1529.94 2359.06 1530.35 Q2357.21 1530.77 2355.26 1531.65 L2355.26 1514.29 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="184.191,1324 2352.76,1324 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="184.191,1007.32 2352.76,1007.32 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="184.191,690.642 2352.76,690.642 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="184.191,373.96 2352.76,373.96 "/>
<polyline clip-path="url(#clip222)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:2; stroke-opacity:0.1; fill:none" points="184.191,57.2787 2352.76,57.2787 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="184.191,1486.45 184.191,47.2441 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="184.191,1324 203.088,1324 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="184.191,1007.32 203.088,1007.32 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="184.191,690.642 203.088,690.642 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="184.191,373.96 203.088,373.96 "/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="184.191,57.2787 203.088,57.2787 "/>
<path clip-path="url(#clip220)" d="M51.6634 1337.35 L59.3023 1337.35 L59.3023 1310.98 L50.9921 1312.65 L50.9921 1308.39 L59.256 1306.72 L63.9319 1306.72 L63.9319 1337.35 L71.5707 1337.35 L71.5707 1341.28 L51.6634 1341.28 L51.6634 1337.35 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M81.0151 1335.4 L85.8993 1335.4 L85.8993 1341.28 L81.0151 1341.28 L81.0151 1335.4 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M110.251 1322.65 Q113.608 1323.37 115.483 1325.64 Q117.381 1327.91 117.381 1331.24 Q117.381 1336.35 113.862 1339.15 Q110.344 1341.96 103.862 1341.96 Q101.686 1341.96 99.3715 1341.52 Q97.0798 1341.1 94.6262 1340.24 L94.6262 1335.73 Q96.5706 1336.86 98.8854 1337.44 Q101.2 1338.02 103.723 1338.02 Q108.121 1338.02 110.413 1336.28 Q112.728 1334.55 112.728 1331.24 Q112.728 1328.18 110.575 1326.47 Q108.446 1324.73 104.626 1324.73 L100.598 1324.73 L100.598 1320.89 L104.811 1320.89 Q108.26 1320.89 110.089 1319.53 Q111.918 1318.14 111.918 1315.54 Q111.918 1312.88 110.02 1311.47 Q108.145 1310.03 104.626 1310.03 Q102.705 1310.03 100.506 1310.45 Q98.3067 1310.87 95.6678 1311.75 L95.6678 1307.58 Q98.3298 1306.84 100.645 1306.47 Q102.983 1306.1 105.043 1306.1 Q110.367 1306.1 113.469 1308.53 Q116.57 1310.94 116.57 1315.06 Q116.57 1317.93 114.927 1319.92 Q113.283 1321.89 110.251 1322.65 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M136.246 1309.8 Q132.635 1309.8 130.807 1313.37 Q129.001 1316.91 129.001 1324.04 Q129.001 1331.15 130.807 1334.71 Q132.635 1338.25 136.246 1338.25 Q139.881 1338.25 141.686 1334.71 Q143.515 1331.15 143.515 1324.04 Q143.515 1316.91 141.686 1313.37 Q139.881 1309.8 136.246 1309.8 M136.246 1306.1 Q142.056 1306.1 145.112 1310.71 Q148.191 1315.29 148.191 1324.04 Q148.191 1332.77 145.112 1337.37 Q142.056 1341.96 136.246 1341.96 Q130.436 1341.96 127.357 1337.37 Q124.302 1332.77 124.302 1324.04 Q124.302 1315.29 127.357 1310.71 Q130.436 1306.1 136.246 1306.1 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M52.6588 1020.67 L60.2976 1020.67 L60.2976 994.302 L51.9875 995.969 L51.9875 991.71 L60.2513 990.043 L64.9272 990.043 L64.9272 1020.67 L72.5661 1020.67 L72.5661 1024.6 L52.6588 1024.6 L52.6588 1020.67 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M82.0105 1018.72 L86.8947 1018.72 L86.8947 1024.6 L82.0105 1024.6 L82.0105 1018.72 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M111.246 1005.97 Q114.603 1006.69 116.478 1008.96 Q118.376 1011.22 118.376 1014.56 Q118.376 1019.67 114.858 1022.47 Q111.339 1025.27 104.858 1025.27 Q102.682 1025.27 100.367 1024.83 Q98.0752 1024.42 95.6215 1023.56 L95.6215 1019.05 Q97.566 1020.18 99.8808 1020.76 Q102.196 1021.34 104.719 1021.34 Q109.117 1021.34 111.408 1019.6 Q113.723 1017.87 113.723 1014.56 Q113.723 1011.5 111.571 1009.79 Q109.441 1008.05 105.621 1008.05 L101.594 1008.05 L101.594 1004.21 L105.807 1004.21 Q109.256 1004.21 111.084 1002.84 Q112.913 1001.46 112.913 998.863 Q112.913 996.2 111.015 994.788 Q109.14 993.353 105.621 993.353 Q103.7 993.353 101.501 993.77 Q99.3021 994.187 96.6632 995.066 L96.6632 990.9 Q99.3252 990.159 101.64 989.788 Q103.978 989.418 106.038 989.418 Q111.362 989.418 114.464 991.849 Q117.566 994.256 117.566 998.376 Q117.566 1001.25 115.922 1003.24 Q114.279 1005.21 111.246 1005.97 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M127.288 990.043 L145.644 990.043 L145.644 993.978 L131.57 993.978 L131.57 1002.45 Q132.589 1002.1 133.607 1001.94 Q134.626 1001.76 135.644 1001.76 Q141.431 1001.76 144.811 1004.93 Q148.191 1008.1 148.191 1013.52 Q148.191 1019.09 144.718 1022.2 Q141.246 1025.27 134.927 1025.27 Q132.751 1025.27 130.482 1024.9 Q128.237 1024.53 125.83 1023.79 L125.83 1019.09 Q127.913 1020.23 130.135 1020.78 Q132.357 1021.34 134.834 1021.34 Q138.839 1021.34 141.177 1019.23 Q143.515 1017.13 143.515 1013.52 Q143.515 1009.9 141.177 1007.8 Q138.839 1005.69 134.834 1005.69 Q132.959 1005.69 131.084 1006.11 Q129.232 1006.52 127.288 1007.4 L127.288 990.043 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M51.6634 703.986 L59.3023 703.986 L59.3023 677.621 L50.9921 679.288 L50.9921 675.028 L59.256 673.362 L63.9319 673.362 L63.9319 703.986 L71.5707 703.986 L71.5707 707.922 L51.6634 707.922 L51.6634 703.986 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M81.0151 702.042 L85.8993 702.042 L85.8993 707.922 L81.0151 707.922 L81.0151 702.042 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M108.932 677.436 L97.1261 695.885 L108.932 695.885 L108.932 677.436 M107.705 673.362 L113.584 673.362 L113.584 695.885 L118.515 695.885 L118.515 699.774 L113.584 699.774 L113.584 707.922 L108.932 707.922 L108.932 699.774 L93.3299 699.774 L93.3299 695.26 L107.705 673.362 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M136.246 676.44 Q132.635 676.44 130.807 680.005 Q129.001 683.547 129.001 690.676 Q129.001 697.783 130.807 701.348 Q132.635 704.889 136.246 704.889 Q139.881 704.889 141.686 701.348 Q143.515 697.783 143.515 690.676 Q143.515 683.547 141.686 680.005 Q139.881 676.44 136.246 676.44 M136.246 672.737 Q142.056 672.737 145.112 677.343 Q148.191 681.926 148.191 690.676 Q148.191 699.403 145.112 704.01 Q142.056 708.593 136.246 708.593 Q130.436 708.593 127.357 704.01 Q124.302 699.403 124.302 690.676 Q124.302 681.926 127.357 677.343 Q130.436 672.737 136.246 672.737 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M52.6588 387.305 L60.2976 387.305 L60.2976 360.939 L51.9875 362.606 L51.9875 358.347 L60.2513 356.68 L64.9272 356.68 L64.9272 387.305 L72.5661 387.305 L72.5661 391.24 L52.6588 391.24 L52.6588 387.305 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M82.0105 385.361 L86.8947 385.361 L86.8947 391.24 L82.0105 391.24 L82.0105 385.361 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M109.927 360.754 L98.1215 379.203 L109.927 379.203 L109.927 360.754 M108.7 356.68 L114.58 356.68 L114.58 379.203 L119.51 379.203 L119.51 383.092 L114.58 383.092 L114.58 391.24 L109.927 391.24 L109.927 383.092 L94.3252 383.092 L94.3252 378.578 L108.7 356.68 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M127.288 356.68 L145.644 356.68 L145.644 360.615 L131.57 360.615 L131.57 369.087 Q132.589 368.74 133.607 368.578 Q134.626 368.393 135.644 368.393 Q141.431 368.393 144.811 371.564 Q148.191 374.736 148.191 380.152 Q148.191 385.731 144.718 388.833 Q141.246 391.911 134.927 391.911 Q132.751 391.911 130.482 391.541 Q128.237 391.171 125.83 390.43 L125.83 385.731 Q127.913 386.865 130.135 387.421 Q132.357 387.976 134.834 387.976 Q138.839 387.976 141.177 385.87 Q143.515 383.763 143.515 380.152 Q143.515 376.541 141.177 374.435 Q138.839 372.328 134.834 372.328 Q132.959 372.328 131.084 372.745 Q129.232 373.162 127.288 374.041 L127.288 356.68 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M51.6634 70.6235 L59.3023 70.6235 L59.3023 44.2579 L50.9921 45.9246 L50.9921 41.6653 L59.256 39.9987 L63.9319 39.9987 L63.9319 70.6235 L71.5707 70.6235 L71.5707 74.5587 L51.6634 74.5587 L51.6634 70.6235 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M81.0151 68.6791 L85.8993 68.6791 L85.8993 74.5587 L81.0151 74.5587 L81.0151 68.6791 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M96.1308 39.9987 L114.487 39.9987 L114.487 43.9338 L100.413 43.9338 L100.413 52.406 Q101.432 52.0588 102.45 51.8967 Q103.469 51.7116 104.487 51.7116 Q110.274 51.7116 113.654 54.8828 Q117.033 58.0541 117.033 63.4708 Q117.033 69.0494 113.561 72.1513 Q110.089 75.23 103.77 75.23 Q101.594 75.23 99.3252 74.8596 Q97.0798 74.4892 94.6724 73.7485 L94.6724 69.0494 Q96.7558 70.1837 98.978 70.7392 Q101.2 71.2948 103.677 71.2948 Q107.682 71.2948 110.02 69.1883 Q112.358 67.0819 112.358 63.4708 Q112.358 59.8597 110.02 57.7532 Q107.682 55.6467 103.677 55.6467 Q101.802 55.6467 99.927 56.0634 Q98.0752 56.4801 96.1308 57.3597 L96.1308 39.9987 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M136.246 43.0774 Q132.635 43.0774 130.807 46.6421 Q129.001 50.1838 129.001 57.3134 Q129.001 64.4198 130.807 67.9846 Q132.635 71.5263 136.246 71.5263 Q139.881 71.5263 141.686 67.9846 Q143.515 64.4198 143.515 57.3134 Q143.515 50.1838 141.686 46.6421 Q139.881 43.0774 136.246 43.0774 M136.246 39.3737 Q142.056 39.3737 145.112 43.9801 Q148.191 48.5634 148.191 57.3134 Q148.191 66.0402 145.112 70.6466 Q142.056 75.23 136.246 75.23 Q130.436 75.23 127.357 70.6466 Q124.302 66.0402 124.302 57.3134 Q124.302 48.5634 127.357 43.9801 Q130.436 39.3737 136.246 39.3737 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><circle clip-path="url(#clip222)" cx="245.565" cy="1445.72" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip222)" cx="1413.65" cy="665.086" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip222)" cx="1820.78" cy="421.125" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<circle clip-path="url(#clip222)" cx="2291.38" cy="87.9763" r="14.4" fill="#009af9" fill-rule="evenodd" fill-opacity="1" stroke="#000000" stroke-opacity="1" stroke-width="3.2"/>
<polyline clip-path="url(#clip222)" style="stroke:#e26f46; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="245.565,1445.39 258.885,1436.59 382.041,1355.29 456.054,1306.43 518.63,1265.12 586.605,1220.24 649.273,1178.87 716.459,1134.52 794.3,1083.13 863.477,1037.46 924.138,997.411 990.276,953.75 1056.81,909.828 1130.36,861.267 1200.35,815.063 1273.12,767.021 1334.38,726.58 1409.17,677.207 1477.68,631.977 1540.96,590.204 1605.06,547.889 1676.89,500.466 1741.98,457.494 1816.51,408.293 1885.52,362.737 1953.55,317.824 2019.6,274.218 2087.32,229.516 2161.5,180.54 2269.27,109.393 2291.38,94.7975 "/>
<path clip-path="url(#clip220)" d="M256.476 198.898 L519.559 198.898 L519.559 95.2176 L256.476 95.2176  Z" fill="#ffffff" fill-rule="evenodd" fill-opacity="1"/>
<polyline clip-path="url(#clip220)" style="stroke:#000000; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="256.476,198.898 519.559,198.898 519.559,95.2176 256.476,95.2176 256.476,198.898 "/>
<polyline clip-path="url(#clip220)" style="stroke:#e26f46; stroke-linecap:round; stroke-linejoin:round; stroke-width:4; stroke-opacity:1; fill:none" points="280.571,147.058 425.142,147.058 "/>
<path clip-path="url(#clip220)" d="M465.742 128.319 L465.742 131.861 L461.668 131.861 Q459.376 131.861 458.474 132.787 Q457.594 133.713 457.594 136.12 L457.594 138.412 L464.608 138.412 L464.608 141.722 L457.594 141.722 L457.594 164.338 L453.312 164.338 L453.312 141.722 L449.238 141.722 L449.238 138.412 L453.312 138.412 L453.312 136.606 Q453.312 132.278 455.325 130.31 Q457.339 128.319 461.714 128.319 L465.742 128.319 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M469.307 138.412 L473.566 138.412 L473.566 164.338 L469.307 164.338 L469.307 138.412 M469.307 128.319 L473.566 128.319 L473.566 133.713 L469.307 133.713 L469.307 128.319 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /><path clip-path="url(#clip220)" d="M486.691 131.051 L486.691 138.412 L495.464 138.412 L495.464 141.722 L486.691 141.722 L486.691 155.796 Q486.691 158.967 487.548 159.87 Q488.427 160.773 491.089 160.773 L495.464 160.773 L495.464 164.338 L491.089 164.338 Q486.159 164.338 484.284 162.509 Q482.409 160.657 482.409 155.796 L482.409 141.722 L479.284 141.722 L479.284 138.412 L482.409 138.412 L482.409 131.051 L486.691 131.051 Z" fill="#000000" fill-rule="nonzero" fill-opacity="1" /></svg>

```

---

*This page was generated using [Literate.jl](https://github.com/fredrikekre/Literate.jl).*

